@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -56,149 +56,96 @@
 #define O_BINARY  0
 #endif
 
-gboolean
-xfdesktop_backdrop_list_is_valid(const gchar *path)
+static GList *
+list_files_in_dir(const gchar *path)
 {
-    FILE *fp;
-    gchar buf[512];
-    gint size;
-    gboolean is_list = FALSE;
+    GDir *dir;
+    gboolean needs_slash = TRUE;
+    const gchar *file;
+    GList *files = NULL;
 
-    size = sizeof(LIST_TEXT);
-
-    if(!(fp = fopen (path, "r")))
-        return FALSE;
-
-    if(fgets(buf, size, fp) && !strncmp(LIST_TEXT, buf, size - 1))
-        is_list = TRUE;
-    fclose(fp);
-
-    return is_list;
-}
-
-gchar **
-xfdesktop_backdrop_list_load(const gchar *filename,
-                             gint *n_items,
-                             GError **error)
-{
-    gchar *contents = NULL, **files = NULL;
-    gsize length = 0;
-    gint i, items;
-
-    g_return_val_if_fail(filename && (!error || !*error), NULL);
-
-    if(!g_file_get_contents(filename, &contents, &length, error))
+    dir = g_dir_open(path, 0, 0);
+    if(!dir)
         return NULL;
 
-    if(strncmp(LIST_TEXT, contents, sizeof(LIST_TEXT) - 1)) {
-        if(error) {
-            g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                        _("Backdrop list file is not valid"));
-        }
-        g_free(contents);
-        return NULL;
+    if(path[strlen(path)-1] == '/')
+        needs_slash = FALSE;
+
+    while((file = g_dir_read_name(dir))) {
+        gchar *current_file = g_strdup_printf(needs_slash ? "%s/%s" : "%s%s",
+                                              path, file);
+
+        files = g_list_insert_sorted(files, current_file, (GCompareFunc)g_strcmp0);
     }
 
-    items = 0;
-    files = g_strsplit(contents, "\n", -1);
-
-    /* Since the first line is the file identifier, we need to skip it.
-     * Additionally, we want to skip blank lines. */
-    for(i = 1; files[i] != NULL; i++) {
-        if(g_strcmp0(files[i], "") != 0) {
-            g_free(files[items]);
-            files[items] = g_strdup(files[i]);
-            DBG("files[items] %s", files[items]);
-            items++;
-        }
-    }
-    files[items+1] = NULL;
-
-    files = g_realloc(files, sizeof(gchar *) * (items+1));
-
-    DBG("items %d", items);
-    if(n_items)
-        *n_items = items;
-
-    g_free(contents);
+    g_dir_close(dir);
 
     return files;
 }
 
-gboolean
-xfdesktop_backdrop_list_save(const gchar *filename,
-                             gchar * const *files,
-                             GError **error)
+
+gchar *
+xfdesktop_backdrop_choose_next(const gchar *filename)
 {
-    gboolean ret = FALSE;
-    gchar *filename_new;
-    FILE *fp;
-    gint i;
+    GList *files, *current_file, *start_file;
+    gchar *file = NULL;
 
-    g_return_val_if_fail(filename && (!error || !*error), FALSE);
+    g_return_val_if_fail(filename, NULL);
 
-    filename_new = g_strconcat(filename, ".new", NULL);
-    fp = fopen(filename_new, "w");
-    if(fp) {
-        fprintf(fp, "%s\n", LIST_TEXT);
-        if(files) {
-            for(i = 0; files[i]; ++i)
-                fprintf(fp, "%s\n", files[i]);
-        }
-        if(!fclose(fp)) {
-            if(!rename(filename_new, filename))
-                ret = TRUE;
-            else  {
-                if(error) {
-                    g_set_error(error, G_FILE_ERROR,
-                                g_file_error_from_errno(errno),
-                                "%s", g_strerror(errno));
-                }
-                unlink(filename_new);
-            }
-        } else {
-            if(error) {
-                g_set_error(error, G_FILE_ERROR, g_file_error_from_errno(errno),
-                            "%s", g_strerror(errno));
-            }
-            unlink(filename_new);
-        }
-    } else if(error) {
-        g_set_error(error, G_FILE_ERROR, g_file_error_from_errno(errno),
-                    "%s", g_strerror(errno));
-    }
+    files = list_files_in_dir(g_path_get_dirname(filename));
+    if(!files)
+        return NULL;
 
-    g_free(filename_new);
+    /* Get the our current background in the list */
+    current_file = g_list_find_custom(files, filename, (GCompareFunc)g_strcmp0);
 
-    return ret;
+    /* if somehow we don't have a valid file, grab the first one available */
+    if(current_file == NULL)
+        current_file = g_list_first(files);
+
+    start_file = current_file;
+
+    /* We want the next valid image file in the dir while making sure
+     * we don't loop on ourselves */
+    do {
+        current_file = g_list_next(current_file);
+
+        /* we hit the end of the list */
+        if(current_file == NULL)
+            current_file = g_list_first(files);
+
+        /* We went through every item in the list */
+        if(g_strcmp0(start_file->data, current_file->data) == 0)
+            break;
+
+    } while(!xfdesktop_image_file_is_valid(current_file->data));
+
+    file = g_strdup(current_file->data);
+    g_list_free_full(files, g_free);
+
+    return file;
 }
 
 gchar *
-xfdesktop_backdrop_list_choose_random(const gchar *filename,
-                                      GError **error)
+xfdesktop_backdrop_choose_random(const gchar *filename)
 {
     static gboolean __initialized = FALSE;
     static gint previndex = -1;
-    gchar **files, *file = NULL;
-    gint n_items = 0, cur_file, i, tries = 0;
+    GList *files;
+    gchar *file = NULL;
+    gint n_items = 0, cur_file, tries = 0;
 
-    g_return_val_if_fail(filename && (!error || !*error), NULL);
+    g_return_val_if_fail(filename, NULL);
 
-    files = xfdesktop_backdrop_list_load(filename, &n_items, error);
+    files = list_files_in_dir(g_path_get_dirname(filename));
     if(!files)
         return NULL;
-    if(!n_items) {
-        if(error) {
-            g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                        _("Backdrop list file is not valid"));
-        }
-        g_strfreev(files);
-        return NULL;
-    }
+
+    n_items = g_list_length(files);
 
     if(1 == n_items) {
-        file = g_strdup(files[0]);
-        g_strfreev(files);
+        file = g_strdup(g_list_first(files)->data);
+        g_list_free_full(files, g_free);
         return file;
     }
 
@@ -219,7 +166,7 @@ xfdesktop_backdrop_list_choose_random(const gchar *filename,
             /* this isn't precise, but if we've failed to get a good
              * image after all this time, let's just give up */
             g_warning("Unable to find good image from list; giving up");
-            g_strfreev(files);
+            g_list_free_full(files, g_free);
             return NULL;
         }
 
@@ -231,65 +178,80 @@ xfdesktop_backdrop_list_choose_random(const gchar *filename,
 #endif
         } while(cur_file == previndex && G_LIKELY(previndex != -1));
 
-    } while(!xfdesktop_image_file_is_valid(files[cur_file]));
+    } while(!xfdesktop_image_file_is_valid(g_list_nth(files, cur_file)->data));
 
     previndex = cur_file;
-    file = files[cur_file];
 
-    /* don't use strfreev() and avoid an extra alloc */
-    for(i = 0; files[i]; ++i) {
-        if(i != cur_file)
-            g_free(files[i]);
-    }
-    g_free(files);
+    file = g_strdup(g_list_nth(files, cur_file)->data);
+    g_list_free_full(files, g_free);
 
     return file;
 }
 
-static void
-pixbuf_loader_size_cb(GdkPixbufLoader *loader, gint width, gint height,
-        gpointer user_data)
+gchar *
+xfdesktop_get_file_mimetype(const gchar *file)
 {
-    gboolean *size_read = user_data;
+    GFile *temp_file;
+    GFileInfo *file_info;
+    gchar *mime_type = NULL;
 
-    if(width > 0 && height > 0)
-        *size_read = TRUE;
+    g_return_val_if_fail(file != NULL, NULL);
+
+    temp_file = g_file_new_for_path(file);
+
+    g_return_val_if_fail(temp_file != NULL, NULL);
+
+    file_info = g_file_query_info(temp_file,
+                                  "standard::content-type",
+                                  0,
+                                  NULL,
+                                  NULL);
+
+    if(file_info != NULL) {
+        mime_type = g_strdup(g_file_info_get_content_type(file_info));
+
+        g_object_unref(file_info);
+    }
+
+    g_object_unref(temp_file);
+
+    return mime_type;
 }
 
 gboolean
 xfdesktop_image_file_is_valid(const gchar *filename)
 {
-    GdkPixbufLoader *loader;
-    int fd;
-    gboolean size_read = FALSE;
-    guchar buf[4096];
-    gssize len;
+    static GSList *pixbuf_formats = NULL;
+    GSList *l;
+    gboolean image_valid = FALSE;
+    gchar *file_mimetype;
 
     g_return_val_if_fail(filename, FALSE);
 
-    fd = open(filename, O_RDONLY|O_BINARY);
-    if(fd < 0)
+    if(pixbuf_formats == NULL) {
+        pixbuf_formats = gdk_pixbuf_get_formats();
+    }
+
+    file_mimetype = xfdesktop_get_file_mimetype(filename);
+
+    if(file_mimetype == NULL)
         return FALSE;
 
-    loader = gdk_pixbuf_loader_new();
-    g_signal_connect(G_OBJECT(loader), "size-prepared",
-            G_CALLBACK(pixbuf_loader_size_cb), &size_read);
+    /* Every pixbuf format has a list of mime types we can compare against */
+    for(l = pixbuf_formats; l != NULL && image_valid == FALSE; l = g_slist_next(l)) {
+        gint i;
+        gchar ** mimetypes = gdk_pixbuf_format_get_mime_types(l->data);
 
-    do {
-        len = read(fd, buf, sizeof(buf));
-        if(len > 0) {
-            if(!gdk_pixbuf_loader_write(loader, buf, len, NULL))
-                break;
-            if(size_read)
-                break;
+        for(i = 0; mimetypes[i] != NULL && image_valid == FALSE; i++) {
+            if(g_strcmp0(file_mimetype, mimetypes[i]) == 0)
+                image_valid = TRUE;
         }
-    } while(len > 0);
+         g_strfreev(mimetypes);
+    }
 
-    close(fd);
-    gdk_pixbuf_loader_close(loader, NULL);
-    g_object_unref(G_OBJECT(loader));
+    g_free(file_mimetype);
 
-    return size_read;
+    return image_valid;
 }
 
 gboolean
@@ -309,9 +271,9 @@ xfdesktop_check_is_running(Window *xid)
         xscreen = 0;
 
     g_snprintf(selection_name, 100, XFDESKTOP_SELECTION_FMT, xscreen);
-    selection_atom = XInternAtom(GDK_DISPLAY(), selection_name, False);
+    selection_atom = XInternAtom(gdk_x11_get_default_xdisplay(), selection_name, False);
 
-    if((*xid = XGetSelectionOwner(GDK_DISPLAY(), selection_atom)))
+    if((*xid = XGetSelectionOwner(gdk_x11_get_default_xdisplay(), selection_atom)))
         return TRUE;
 
     return FALSE;
@@ -327,7 +289,7 @@ xfdesktop_send_client_message(Window xid, const gchar *msg)
     gtk_widget_realize(win);
 
     gev.type = GDK_CLIENT_EVENT;
-    gev.window = win->window;
+    gev.window = gtk_widget_get_window(win);
     gev.send_event = TRUE;
     gev.message_type = gdk_atom_intern("STRING", FALSE);
     gev.data_format = 8;
@@ -339,6 +301,36 @@ xfdesktop_send_client_message(Window xid, const gchar *msg)
     gtk_widget_destroy(win);
 }
 
+guint
+xfce_grab_cursor(GtkWidget *w,
+                 GdkEventButton *evt)
+{
+    GdkCursor     *cursor;
+    GdkGrabStatus  status;
+    GdkDisplay    *display;
+
+    TRACE("entering");
+
+    /* create a cursor */
+    display = gdk_screen_get_display(gtk_widget_get_screen(w));
+    cursor = gdk_cursor_new_for_display(display, GDK_FLEUR);
+
+    /* grab the pointer for the desktop */
+    status = gdk_pointer_grab(evt->window, FALSE,
+                              GDK_BUTTON_MOTION_MASK
+                              | GDK_BUTTON_RELEASE_MASK,
+                              NULL, cursor, evt->time);
+
+    gdk_cursor_unref(cursor);
+
+    if(status != GDK_GRAB_SUCCESS) {
+        g_warning("mouse grab failed.");
+        return 0;
+    }
+
+    return evt->time;
+}
+
 /* Code taken from xfwm4/src/menu.c:grab_available().  This should fix the case
  * where binding 'xfdesktop -menu' to a keyboard shortcut sometimes works and
  * sometimes doesn't.  Credit for this one goes to Olivier.
