@@ -16,8 +16,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  xfdesktop-thumbnailer is based on thumbnailer code from Ristretto
  *  Copyright (c) Stephan Arts 2009-2011 <stephan@xfce.org>
@@ -41,6 +41,7 @@
 #include <libxfce4util/libxfce4util.h>
 #include "xfdesktop-thumbnailer.h"
 #include "xfdesktop-marshal.h"
+#include "xfdesktop-common.h"
 
 static void xfdesktop_thumbnailer_init(GObject *);
 static void xfdesktop_thumbnailer_class_init(GObjectClass *);
@@ -257,34 +258,14 @@ xfdesktop_thumbnailer_new(void)
     return thumbnailer_object;
 }
 
-static gchar *
-xfdesktop_get_file_mimetype(gchar *file)
+gboolean xfdesktop_thumbnailer_service_available(XfdesktopThumbnailer *thumbnailer)
 {
-    GFile *temp_file;
-    GFileInfo *file_info;
-    gchar *mime_type = NULL;
+    g_return_val_if_fail(XFDESKTOP_IS_THUMBNAILER(thumbnailer), FALSE);
 
-    g_return_val_if_fail(file != NULL, NULL);
+    if(thumbnailer->priv->proxy == NULL)
+        return FALSE;
 
-    temp_file = g_file_new_for_path(file);
-
-    g_return_val_if_fail(temp_file != NULL, NULL);
-
-    file_info = g_file_query_info(temp_file,
-                                  "standard::content-type",
-                                  0,
-                                  NULL,
-                                  NULL);
-
-    if(file_info != NULL) {
-        mime_type = g_strdup(g_file_info_get_content_type(file_info));
-
-        g_object_unref(file_info);
-    }
-
-    g_object_unref(temp_file);
-    
-    return mime_type;
+    return TRUE;
 }
 
 gboolean
@@ -370,6 +351,12 @@ xfdesktop_thumbnailer_queue_thumbnail(XfdesktopThumbnailer *thumbnailer,
     return TRUE;
 }
 
+static void
+xfdesktop_thumbnailer_dequeue_foreach(gpointer data, gpointer user_data)
+{
+    xfdesktop_thumbnailer_dequeue_thumbnail(user_data, data);
+}
+
 /**
  * xfdesktop_thumbnailer_dequeue_thumbnail:
  * 
@@ -402,9 +389,8 @@ xfdesktop_thumbnailer_dequeue_thumbnail(XfdesktopThumbnailer *thumbnailer,
     }
 
     if(g_slist_find(thumbnailer->priv->queue, file) != NULL) {
-            thumbnailer->priv->queue = g_slist_remove_all(
-                                                    thumbnailer->priv->queue,
-                                                    file);
+        thumbnailer->priv->queue = g_slist_remove_all(thumbnailer->priv->queue,
+                                                      file);
     }
 
     thumbnailer->priv->request_timer_id = g_timeout_add_full(
@@ -415,6 +401,13 @@ xfdesktop_thumbnailer_dequeue_thumbnail(XfdesktopThumbnailer *thumbnailer,
                         NULL);
 }
 
+void xfdesktop_thumbnailer_dequeue_all_thumbnails(XfdesktopThumbnailer *thumbnailer)
+{
+    g_return_if_fail(XFDESKTOP_IS_THUMBNAILER(thumbnailer));
+
+    g_slist_foreach(thumbnailer->priv->queue, (GFunc)xfdesktop_thumbnailer_dequeue_foreach, thumbnailer);
+}
+
 static gboolean
 xfdesktop_thumbnailer_queue_request_timer(XfdesktopThumbnailer *thumbnailer)
 {
@@ -517,11 +510,7 @@ xfdesktop_thumbnailer_thumbnail_ready_dbus(DBusGProxy *proxy,
 
         if(strcmp (uri[x], f_uri) == 0) {
             /* The thumbnail is in the format/location
-             * $XDG_CACHE_HOME/thumbnails/(nromal|large)/MD5_Hash_Of_URI.png
-             * for version 0.8.0 if XDG_CACHE_HOME is defined, otherwise
              * /homedir/.thumbnails/(normal|large)/MD5_Hash_Of_URI.png
-             * will be used, which is also always used for versions prior
-             * to 0.7.0.
              */
             f_uri_checksum = g_compute_checksum_for_string(G_CHECKSUM_MD5,
                                                            f_uri, strlen (f_uri));
@@ -533,31 +522,19 @@ xfdesktop_thumbnailer_thumbnail_ready_dbus(DBusGProxy *proxy,
 
             filename = g_strconcat(f_uri_checksum, ".png", NULL);
 
-            /* build and check if the thumbnail is in the new location */
-            thumbnail_location = g_build_path("/", g_get_user_cache_dir(),
-                                              "thumbnails", thumbnail_flavor,
+            thumbnail_location = g_build_path("/", g_get_home_dir(),
+                                              ".thumbnails", thumbnail_flavor,
                                               filename, NULL);
 
-            if(!g_file_test(thumbnail_location, G_FILE_TEST_EXISTS)) {
-                /* Fallback to old version */
-                g_free(thumbnail_location);
-
-                thumbnail_location = g_build_path("/", g_get_home_dir(),
-                                                  ".thumbnails", thumbnail_flavor,
-                                                  filename, NULL);
-            }
-
             DBG("thumbnail-ready src: %s thumbnail: %s",
                     (char*)iter->data,
                     thumbnail_location);
 
-            if(g_file_test(thumbnail_location, G_FILE_TEST_EXISTS)) {
-                g_signal_emit(G_OBJECT(thumbnailer),
-                              thumbnailer_signals[THUMBNAIL_READY],
-                              0,
-                              iter->data,
-                              thumbnail_location);
-            }
+            g_signal_emit(G_OBJECT(thumbnailer),
+                          thumbnailer_signals[THUMBNAIL_READY],
+                          0,
+                          iter->data,
+                          thumbnail_location);
 
             thumbnailer->priv->queue = g_slist_remove(thumbnailer->priv->queue,
                                                       iter->data);
