@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  Random portions taken from or inspired by the original xfdesktop for xfce4:
  *     Copyright (C) 2002-2003 Jasper Huijsmans (huysmans@users.sourceforge.net)
@@ -182,12 +182,10 @@ client_message_received(GtkWidget *w, GdkEventClient *evt, gpointer user_data)
             g_idle_add((GSourceFunc)reload_idle_cb, user_data);
             return TRUE;
         } else if(!strcmp(MENU_MESSAGE, evt->data.b)) {
-            xfce_desktop_popup_root_menu(XFCE_DESKTOP(w), 0,
-                                         GDK_CURRENT_TIME);
+            xfce_desktop_popup_root_menu(XFCE_DESKTOP(w), 0, 0);
             return TRUE;
         } else if(!strcmp(WINDOWLIST_MESSAGE, evt->data.b)) {
-            xfce_desktop_popup_secondary_root_menu(XFCE_DESKTOP(w), 0,
-                                                   GDK_CURRENT_TIME);
+            xfce_desktop_popup_secondary_root_menu(XFCE_DESKTOP(w), 0, 0);
             return TRUE;
 #ifdef ENABLE_FILE_ICONS
         } else if(!strcmp(ARRANGE_MESSAGE, evt->data.b)) {
@@ -253,12 +251,7 @@ main(int argc, char **argv)
     
     /* bind gettext textdomain */
     xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
-    
-#if defined(ENABLE_FILE_ICONS) || defined(USE_DESKTOP_MENU)
-#if !GLIB_CHECK_VERSION(2, 32, 0)
-    g_thread_init(NULL);
-#endif /* GLIB_CHECK_VERSION */
-#endif
+
 #ifdef ENABLE_FILE_ICONS
     dbus_g_thread_init();
 #endif
@@ -376,7 +369,7 @@ main(int argc, char **argv)
         menu_attach(XFCE_DESKTOP(desktops[i]));
         windowlist_attach(XFCE_DESKTOP(desktops[i]));
         gtk_widget_show(desktops[i]);
-        gdk_window_lower(desktops[i]->window);
+        gdk_window_lower(gtk_widget_get_window(desktops[i]));
     }
     
     for(i = 0; i < nscreens; ++i) {
