@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  Random portions taken from or inspired by the original xfdesktop for xfce4:
  *     Copyright (C) 2002-2003 Jasper Huijsmans (huysmans@users.sourceforge.net)
@@ -45,6 +45,10 @@
 #define abs(x)  ( (x) < 0 ? -(x) : (x) )
 #endif
 
+#define XFCE_BACKDROP_BUFFER_SIZE 4096
+
+typedef struct _XfceBackdropImageData XfceBackdropImageData;
+
 static void xfce_backdrop_finalize(GObject *object);
 static void xfce_backdrop_set_property(GObject *object,
                                        guint property_id,
@@ -56,32 +60,63 @@ static void xfce_backdrop_get_property(GObject *object,
                                        GParamSpec *pspec);
 static gboolean xfce_backdrop_timer(XfceBackdrop *backdrop);
 
+static GdkPixbuf *xfce_backdrop_generate_canvas(XfceBackdrop *backdrop);
+
+static void xfce_backdrop_loader_size_prepared_cb(GdkPixbufLoader *loader,
+                                                  gint width,
+                                                  gint height,
+                                                  gpointer user_data);
+
+static void xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
+                                           XfceBackdropImageData *image_data);
+
+static void xfce_backdrop_file_ready_cb(GObject *source_object,
+                                        GAsyncResult *res,
+                                        gpointer user_data);
+
+static void xfce_backdrop_file_input_stream_ready_cb(GObject *source_object,
+                                                     GAsyncResult *res,
+                                                     gpointer user_data);
+
+static void xfce_backdrop_image_data_release(XfceBackdropImageData *image_data);
+
 struct _XfceBackdropPriv
 {
     gint width, height;
     gint bpp;
-    
+
+    GdkPixbuf *pix;
+    XfceBackdropImageData *image_data;
+
     XfceBackdropColorStyle color_style;
     GdkColor color1;
     GdkColor color2;
-    
-    gboolean show_image;
+
     XfceBackdropImageStyle image_style;
     gchar *image_path;
-    gchar *backdrop_list;
-    
-    gint brightness;
-    gdouble saturation;
 
     gboolean cycle_backdrop;
     guint cycle_timer;
     guint cycle_timer_id;
+    gboolean random_backdrop_order;
+};
+
+struct _XfceBackdropImageData
+{
+    XfceBackdrop *backdrop;
+
+    GdkPixbufLoader *loader;
+
+    GCancellable *cancellable;
+
+    guchar *image_buffer;
 };
 
 enum
 {
     BACKDROP_CHANGED,
     BACKDROP_CYCLE,
+    BACKDROP_READY,
     LAST_SIGNAL,
 };
 
@@ -91,65 +126,18 @@ enum
     PROP_COLOR_STYLE,
     PROP_COLOR1,
     PROP_COLOR2,
-    PROP_SHOW_IMAGE,
     PROP_IMAGE_STYLE,
     PROP_IMAGE_FILENAME,
     PROP_BRIGHTNESS,
-    PROP_SATURATION,
     PROP_BACKDROP_CYCLE_ENABLE,
     PROP_BACKDROP_CYCLE_TIMER,
+    PROP_BACKDROP_RANDOM_ORDER
 };
 
 static guint backdrop_signals[LAST_SIGNAL] = { 0, };
 
 /* helper functions */
 
-static GdkPixbuf *
-adjust_brightness(GdkPixbuf *src, gint amount)
-{
-    GdkPixbuf *newpix;
-    GdkPixdata pdata;
-    gboolean has_alpha = FALSE;
-    gint i, len;
-    GError *err = NULL;
-    
-    g_return_val_if_fail(src != NULL, NULL);
-    if(amount == 0)
-        return src;
-    
-    gdk_pixdata_from_pixbuf(&pdata, src, FALSE);
-    has_alpha = (pdata.pixdata_type & GDK_PIXDATA_COLOR_TYPE_RGBA);
-    if(pdata.length < 1)
-        len = pdata.width * pdata.height * (has_alpha?4:3);
-    else
-        len = pdata.length - GDK_PIXDATA_HEADER_LENGTH;
-    
-    for(i = 0; i < len; i++) {
-        gshort scaled;
-        
-        if(has_alpha && (i+1)%4)
-            continue;
-        
-        scaled = pdata.pixel_data[i] + amount;
-        if(scaled > 255)
-            scaled = 255;
-        if(scaled < 0)
-            scaled = 0;
-        pdata.pixel_data[i] = scaled;
-    }
-    
-    newpix = gdk_pixbuf_from_pixdata(&pdata, TRUE, &err);
-    if(!newpix) {
-        g_warning("%s: Unable to modify image brightness: %s", PACKAGE,
-                err->message);
-        g_error_free(err);
-        return src;
-    }
-    g_object_unref(G_OBJECT(src));
-    
-    return newpix;
-}
-
 static GdkPixbuf *
 create_solid(GdkColor *color,
              gint width,
@@ -230,6 +218,19 @@ create_gradient(GdkColor *color1, GdkColor *color2, gint width, gint height,
     return pix;
 }
 
+static void
+xfce_backdrop_clear_cached_image(XfceBackdrop *backdrop)
+{
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    if(backdrop->priv->pix == NULL)
+        return;
+
+    g_object_unref(backdrop->priv->pix);
+    backdrop->priv->pix = NULL;
+}
+
+
 /* gobject-related functions */
 
 
@@ -255,6 +256,10 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
             G_OBJECT_CLASS_TYPE(gobject_class), G_SIGNAL_RUN_FIRST,
             G_STRUCT_OFFSET(XfceBackdropClass, cycle), NULL, NULL,
             g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+    backdrop_signals[BACKDROP_READY] = g_signal_new("ready",
+            G_OBJECT_CLASS_TYPE(gobject_class), G_SIGNAL_RUN_FIRST,
+            G_STRUCT_OFFSET(XfceBackdropClass, ready), NULL, NULL,
+            g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 #define XFDESKTOP_PARAM_FLAGS  (G_PARAM_READWRITE \
                                 | G_PARAM_CONSTRUCT \
@@ -284,19 +289,12 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
                                                        GDK_TYPE_COLOR,
                                                        XFDESKTOP_PARAM_FLAGS));
 
-    g_object_class_install_property(gobject_class, PROP_SHOW_IMAGE,
-                                    g_param_spec_boolean("show-image",
-                                                         "show image",
-                                                         "show image",
-                                                         TRUE,
-                                                         XFDESKTOP_PARAM_FLAGS));
-
     g_object_class_install_property(gobject_class, PROP_IMAGE_STYLE,
                                     g_param_spec_enum("image-style",
                                                       "image style",
                                                       "image style",
                                                       XFCE_TYPE_BACKDROP_IMAGE_STYLE,
-                                                      XFCE_BACKDROP_IMAGE_AUTO,
+                                                      XFCE_BACKDROP_IMAGE_SCALED,
                                                       XFDESKTOP_PARAM_FLAGS));
 
     g_object_class_install_property(gobject_class, PROP_IMAGE_FILENAME,
@@ -306,20 +304,6 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
                                                         DEFAULT_BACKDROP,
                                                         XFDESKTOP_PARAM_FLAGS));
 
-    g_object_class_install_property(gobject_class, PROP_BRIGHTNESS,
-                                    g_param_spec_int("brightness",
-                                                     "brightness",
-                                                     "brightness",
-                                                     -128, 127, 0,
-                                                     XFDESKTOP_PARAM_FLAGS));
-
-    g_object_class_install_property(gobject_class, PROP_SATURATION,
-                                    g_param_spec_double("saturation",
-                                                        "saturation",
-                                                        "saturation",
-                                                        -10.0, 10.0, 1.0,
-                                                        XFDESKTOP_PARAM_FLAGS));
-
     g_object_class_install_property(gobject_class, PROP_BACKDROP_CYCLE_ENABLE,
                                     g_param_spec_boolean("backdrop-cycle-enable",
                                                          "backdrop-cycle-enable",
@@ -334,6 +318,13 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
                                                      0, G_MAXUSHORT, 10,
                                                      XFDESKTOP_PARAM_FLAGS));
 
+    g_object_class_install_property(gobject_class, PROP_BACKDROP_RANDOM_ORDER,
+                                    g_param_spec_boolean("backdrop-cycle-random-order",
+                                                         "backdrop-cycle-random-order",
+                                                         "backdrop-cycle-random-order",
+                                                         FALSE,
+                                                         XFDESKTOP_PARAM_FLAGS));
+
 #undef XFDESKTOP_PARAM_FLAGS
 }
 
@@ -342,9 +333,7 @@ xfce_backdrop_init(XfceBackdrop *backdrop)
 {
     backdrop->priv = G_TYPE_INSTANCE_GET_PRIVATE(backdrop, XFCE_TYPE_BACKDROP,
                                                  XfceBackdropPriv);
-    backdrop->priv->show_image = TRUE;
     backdrop->priv->cycle_timer_id = 0;
-    backdrop->priv->backdrop_list = NULL;
 
     /* color defaults */
     backdrop->priv->color1.red = 0x1515;
@@ -370,10 +359,7 @@ xfce_backdrop_finalize(GObject *object)
         backdrop->priv->cycle_timer_id = 0;
     }
 
-    if(backdrop->priv->backdrop_list != NULL) {
-        g_free(backdrop->priv->backdrop_list);
-        backdrop->priv->backdrop_list = NULL;
-    }
+    xfce_backdrop_clear_cached_image(backdrop);
 
     G_OBJECT_CLASS(xfce_backdrop_parent_class)->finalize(object);
 }
@@ -404,10 +390,6 @@ xfce_backdrop_set_property(GObject *object,
                 xfce_backdrop_set_second_color(backdrop, color);
             break;
 
-        case PROP_SHOW_IMAGE:
-            xfce_backdrop_set_show_image(backdrop, g_value_get_boolean(value));
-            break;
-
         case PROP_IMAGE_STYLE:
             xfce_backdrop_set_image_style(backdrop, g_value_get_enum(value));
             break;
@@ -417,14 +399,6 @@ xfce_backdrop_set_property(GObject *object,
                                              g_value_get_string(value));
             break;
 
-        case PROP_BRIGHTNESS:
-            xfce_backdrop_set_brightness(backdrop, g_value_get_int(value));
-            break;
-
-        case PROP_SATURATION:
-            xfce_backdrop_set_saturation(backdrop, g_value_get_double(value));
-            break;
-
         case PROP_BACKDROP_CYCLE_ENABLE:
             xfce_backdrop_set_cycle_backdrop(backdrop, g_value_get_boolean(value));
             break;
@@ -433,6 +407,10 @@ xfce_backdrop_set_property(GObject *object,
             xfce_backdrop_set_cycle_timer(backdrop, g_value_get_uint(value));
             break;
 
+        case PROP_BACKDROP_RANDOM_ORDER:
+            xfce_backdrop_set_random_order(backdrop, g_value_get_boolean(value));
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
             break;
@@ -460,10 +438,6 @@ xfce_backdrop_get_property(GObject *object,
             g_value_set_boxed(value, &backdrop->priv->color2);
             break;
 
-        case PROP_SHOW_IMAGE:
-            g_value_set_boolean(value, xfce_backdrop_get_show_image(backdrop));
-            break;
-
         case PROP_IMAGE_STYLE:
             g_value_set_enum(value, xfce_backdrop_get_image_style(backdrop));
             break;
@@ -473,14 +447,6 @@ xfce_backdrop_get_property(GObject *object,
                                xfce_backdrop_get_image_filename(backdrop));
             break;
 
-        case PROP_BRIGHTNESS:
-            g_value_set_int(value, xfce_backdrop_get_brightness(backdrop));
-            break;
-
-        case PROP_SATURATION:
-            g_value_set_double(value, xfce_backdrop_get_saturation(backdrop));
-            break;
-
         case PROP_BACKDROP_CYCLE_ENABLE:
             g_value_set_boolean(value, xfce_backdrop_get_cycle_backdrop(backdrop));
             break;
@@ -489,6 +455,10 @@ xfce_backdrop_get_property(GObject *object,
             g_value_set_uint(value, xfce_backdrop_get_cycle_timer(backdrop));
             break;
 
+        case PROP_BACKDROP_RANDOM_ORDER:
+            g_value_set_boolean(value, xfce_backdrop_get_random_order(backdrop));
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
             break;
@@ -516,7 +486,7 @@ xfce_backdrop_new(GdkVisual *visual)
     g_return_val_if_fail(GDK_IS_VISUAL(visual), NULL);
     
     backdrop = g_object_new(XFCE_TYPE_BACKDROP, NULL);
-    backdrop->priv->bpp = visual->depth;
+    backdrop->priv->bpp = gdk_visual_get_depth(visual);
     
     return backdrop;
 }
@@ -541,7 +511,7 @@ xfce_backdrop_new_with_size(GdkVisual *visual,
     
     backdrop = g_object_new(XFCE_TYPE_BACKDROP, NULL);
     
-    backdrop->priv->bpp = visual->depth;
+    backdrop->priv->bpp = gdk_visual_get_depth(visual);
     backdrop->priv->width = width;
     backdrop->priv->height = height;
 
@@ -562,9 +532,13 @@ void
 xfce_backdrop_set_size(XfceBackdrop *backdrop, gint width, gint height)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
-    
-    backdrop->priv->width = width;
-    backdrop->priv->height = height;
+
+    if(backdrop->priv->width != width ||
+       backdrop->priv->height != height) {
+        xfce_backdrop_clear_cached_image(backdrop);
+        backdrop->priv->width = width;
+        backdrop->priv->height = height;
+    }
 }
 
 /**
@@ -582,6 +556,7 @@ xfce_backdrop_set_color_style(XfceBackdrop *backdrop,
     g_return_if_fail((int)style >= 0 && style <= XFCE_BACKDROP_COLOR_TRANSPARENT);
     
     if(style != backdrop->priv->color_style) {
+        xfce_backdrop_clear_cached_image(backdrop);
         backdrop->priv->color_style = style;
         g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
     }
@@ -616,6 +591,7 @@ xfce_backdrop_set_first_color(XfceBackdrop *backdrop,
             || color->green != backdrop->priv->color1.green
             || color->blue != backdrop->priv->color1.blue)
     {
+        xfce_backdrop_clear_cached_image(backdrop);
         backdrop->priv->color1.red = color->red;
         backdrop->priv->color1.green = color->green;
         backdrop->priv->color1.blue = color->blue;
@@ -652,6 +628,7 @@ xfce_backdrop_set_second_color(XfceBackdrop *backdrop,
             || color->green != backdrop->priv->color2.green
             || color->blue != backdrop->priv->color2.blue)
     {
+        xfce_backdrop_clear_cached_image(backdrop);
         backdrop->priv->color2.red = color->red;
         backdrop->priv->color2.green = color->green;
         backdrop->priv->color2.blue = color->blue;
@@ -669,32 +646,6 @@ xfce_backdrop_get_second_color(XfceBackdrop *backdrop,
     memcpy(color, &backdrop->priv->color2, sizeof(GdkColor));
 }
 
-/**
- * xfce_backdrop_set_show_image:
- * @backdrop: An #XfceBackdrop.
- * @show_image: Whether or not to composite the image on top of the color.
- *
- * Sets whether or not the #XfceBackdrop should consist of an image composited
- * on top of a color (or color gradient), or just a color (or color gradient).
- **/
-void
-xfce_backdrop_set_show_image(XfceBackdrop *backdrop, gboolean show_image)
-{
-    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
-    
-    if(backdrop->priv->show_image != show_image) {
-        backdrop->priv->show_image = show_image;
-        g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
-    }
-}
-
-gboolean
-xfce_backdrop_get_show_image(XfceBackdrop *backdrop)
-{
-    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), FALSE);
-    return backdrop->priv->show_image;
-}
-
 /**
  * xfce_backdrop_set_image_style:
  * @backdrop: An #XfceBackdrop.
@@ -713,6 +664,7 @@ xfce_backdrop_set_image_style(XfceBackdrop *backdrop,
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
     
     if(style != backdrop->priv->image_style) {
+        xfce_backdrop_clear_cached_image(backdrop);
         backdrop->priv->image_style = style;
         g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
     }
@@ -739,14 +691,18 @@ void
 xfce_backdrop_set_image_filename(XfceBackdrop *backdrop, const gchar *filename)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
-    
+
+    TRACE("entering");
+
     g_free(backdrop->priv->image_path);
     
     if(filename)
         backdrop->priv->image_path = g_strdup(filename);
     else
         backdrop->priv->image_path = NULL;
-    
+
+    xfce_backdrop_clear_cached_image(backdrop);
+
     g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
 }
 
@@ -757,82 +713,6 @@ xfce_backdrop_get_image_filename(XfceBackdrop *backdrop)
     return backdrop->priv->image_path;
 }
 
-/**
- * xfce_backdrop_set_list:
- * @backdrop: An #XfceBackdrop.
- * @backdrop_list: A list of backdrop images.
- *
- * Sets the internal list of wallpaper images assigned to this backdrop.
- * Frees any previous backdrop list before setting the new one.
- * [Transfer full]
- **/
-void
-xfce_backdrop_set_list(XfceBackdrop *backdrop,
-                       gchar *backdrop_list)
-{
-    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
-
-    if(backdrop->priv->backdrop_list != NULL) {
-        g_free(backdrop->priv->backdrop_list);
-        backdrop->priv->backdrop_list = NULL;
-    }
-
-    backdrop->priv->backdrop_list = backdrop_list;
-}
-
-G_CONST_RETURN gchar*
-xfce_backdrop_get_list(XfceBackdrop *backdrop)
-{
-    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), NULL);
-
-    return backdrop->priv->backdrop_list;
-}
-
-/**
- * xfce_backdrop_set_brightness:
- * @backdrop: An #XfceBackdrop.
- * @brightness: A brightness value.
- *
- * Modifies the brightness of the backdrop using a value between -128 and 127.
- * A value of 0 indicates that the brightness should not be changed.  This value
- * is applied to the entire image, after compositing.
- **/
-void
-xfce_backdrop_set_brightness(XfceBackdrop *backdrop, gint brightness)
-{
-    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
-    
-    if(brightness != backdrop->priv->brightness) {
-        backdrop->priv->brightness = brightness;
-        g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
-    }
-}
-
-gint
-xfce_backdrop_get_brightness(XfceBackdrop *backdrop)
-{
-    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), 0);
-    return backdrop->priv->brightness;
-}
-
-void
-xfce_backdrop_set_saturation(XfceBackdrop *backdrop,
-                             gdouble saturation)
-{
-    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
-    if(abs(saturation - backdrop->priv->saturation) > 0.05) {
-        backdrop->priv->saturation = saturation;
-        g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
-    }
-}
-
-gdouble
-xfce_backdrop_get_saturation(XfceBackdrop *backdrop)
-{
-    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), 1.0);
-    return backdrop->priv->saturation;
-}
-
 static gboolean
 xfce_backdrop_timer(XfceBackdrop *backdrop)
 {
@@ -840,7 +720,9 @@ xfce_backdrop_timer(XfceBackdrop *backdrop)
 
     g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), FALSE);
 
-    g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CYCLE], 0);
+    /* Don't bother with trying to cycle a backdrop if we're not using images */
+    if(backdrop->priv->image_style != XFCE_BACKDROP_IMAGE_NONE)
+        g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CYCLE], 0);
 
     return TRUE;
 }
@@ -911,45 +793,34 @@ xfce_backdrop_get_cycle_backdrop(XfceBackdrop *backdrop)
     return backdrop->priv->cycle_backdrop;
 }
 
-/**
- * xfce_backdrop_get_pixbuf:
- * @backdrop: An #XfceBackdrop.
- *
- * Generates the final composited, resized image from the #XfceBackdrop.  Free
- * it with g_object_unref() when you are finished.
- *
- * Return value: A #GdkPixbuf.
- **/
-GdkPixbuf *
-xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
+void
+xfce_backdrop_set_random_order(XfceBackdrop *backdrop,
+                               gboolean random_order)
 {
-    GdkPixbuf *final_image, *image = NULL, *tmp;
-    GdkPixbufFormat *format = NULL;
-    gboolean apply_backdrop_image = FALSE;
-    gint i, j;
-    gint w, h, iw = 0, ih = 0;
-    XfceBackdropImageStyle istyle;
-    gint dx, dy, xo, yo;
-    gdouble xscale, yscale;
-    GdkInterpType interp;
-    
-    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), NULL);
-    
-    if(backdrop->priv->show_image && backdrop->priv->image_path) {
-        format = gdk_pixbuf_get_file_info(backdrop->priv->image_path, &iw, &ih);
-        /* make sure we have a usable backdrop image */
-        if(format != NULL)
-            apply_backdrop_image = TRUE;
-    }
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    TRACE("entering");
+
+    backdrop->priv->random_backdrop_order = random_order;
+}
+
+gboolean
+xfce_backdrop_get_random_order(XfceBackdrop *backdrop)
+{
+    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), FALSE);
+
+    return backdrop->priv->random_backdrop_order;
+}
+
+static GdkPixbuf *
+xfce_backdrop_generate_canvas(XfceBackdrop *backdrop)
+{
+    gint w, h;
+    GdkPixbuf *final_image;
+
+    w = backdrop->priv->width;
+    h = backdrop->priv->height;
 
-    if(backdrop->priv->width == 0 || backdrop->priv->height == 0) {
-        w = iw;
-        h = ih;
-    } else {
-        w = backdrop->priv->width;
-        h = backdrop->priv->height;
-    }
-    
     if(backdrop->priv->color_style == XFCE_BACKDROP_COLOR_SOLID)
         final_image = create_solid(&backdrop->priv->color1, w, h, FALSE, 0xff);
     else if(backdrop->priv->color_style == XFCE_BACKDROP_COLOR_TRANSPARENT) {
@@ -961,21 +832,195 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
         if(!final_image)
             final_image = create_solid(&backdrop->priv->color1, w, h, FALSE, 0xff);
     }
-    
-    if(!apply_backdrop_image) {
-        if(backdrop->priv->brightness != 0)
-            final_image = adjust_brightness(final_image, backdrop->priv->brightness);
-        
-        return final_image;
+
+    return final_image;
+}
+
+static void
+xfce_backdrop_image_data_release(XfceBackdropImageData *image_data)
+{
+    TRACE("entering");
+
+    if(!image_data)
+        return;
+
+    /* Only set the backdrop's image_data to NULL if it's current */
+    if(image_data->backdrop->priv->image_data == image_data)
+        image_data->backdrop->priv->image_data = NULL;
+
+    if(image_data->cancellable)
+        g_object_unref(image_data->cancellable);
+
+    if(image_data->image_buffer)
+        g_free(image_data->image_buffer);
+
+    if(image_data->loader)
+        g_object_unref(image_data->loader);
+
+    g_free(image_data);
+    image_data = NULL;
+
+}
+
+/* Returns the composited backdrop image if one has been generated. If it
+ * returns NULL, call xfce_backdrop_generate_async to create the pixbuf.
+ * Free with g_object_unref() when you are finished. */
+GdkPixbuf *
+xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
+{
+    TRACE("entering");
+
+    if(backdrop->priv->pix)
+        return g_object_ref(backdrop->priv->pix);
+
+    return NULL;
+}
+
+/* Generates the final composited, resized image from the #XfceBackdrop.
+ * Emits the "ready" signal when the image has been created */
+void
+xfce_backdrop_generate_async(XfceBackdrop *backdrop)
+{
+    GFile *file;
+    XfceBackdropImageData *image_data = NULL;
+
+    TRACE("entering");
+
+    if(backdrop->priv->width == 0 || backdrop->priv->height == 0) {
+        g_critical("attempting to create a backdrop without setting the width/height");
+        return;
+    }
+
+    if(backdrop->priv->image_data && backdrop->priv->image_data->cancellable) {
+        g_cancellable_cancel(backdrop->priv->image_data->cancellable);
+        backdrop->priv->image_data = NULL;
+    }
+
+    /* If we aren't going to display an image then just create the canvas */
+    if(backdrop->priv->image_style == XFCE_BACKDROP_IMAGE_NONE
+       || !backdrop->priv->image_path) {
+        backdrop->priv->pix = xfce_backdrop_generate_canvas(backdrop);
+        g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_READY], 0);
+        return;
+    }
+
+    file = g_file_new_for_path(backdrop->priv->image_path);
+    image_data = g_new0(XfceBackdropImageData, 1);
+
+    backdrop->priv->image_data = image_data;
+
+    image_data->backdrop = backdrop;
+    image_data->loader = gdk_pixbuf_loader_new();
+    image_data->cancellable = g_cancellable_new();
+    image_data->image_buffer = g_new0(guchar, XFCE_BACKDROP_BUFFER_SIZE);
+
+    g_signal_connect(image_data->loader, "size-prepared", G_CALLBACK(xfce_backdrop_loader_size_prepared_cb), image_data);
+    g_signal_connect(image_data->loader, "closed", G_CALLBACK(xfce_backdrop_loader_closed_cb), image_data);
+
+    g_file_read_async(file,
+                      G_PRIORITY_LOW,
+                      image_data->cancellable,
+                      xfce_backdrop_file_ready_cb,
+                      image_data);
+}
+
+
+static void
+xfce_backdrop_loader_size_prepared_cb(GdkPixbufLoader *loader,
+                                      gint width,
+                                      gint height,
+                                      gpointer user_data)
+{
+    XfceBackdropImageData *image_data = user_data;
+    XfceBackdrop *backdrop = image_data->backdrop;
+    gdouble xscale, yscale;
+
+    TRACE("entering");
+
+    switch(backdrop->priv->image_style) {
+        case XFCE_BACKDROP_IMAGE_CENTERED:
+        case XFCE_BACKDROP_IMAGE_TILED:
+            /* do nothing */
+            break;
+
+        case XFCE_BACKDROP_IMAGE_STRETCHED:
+            gdk_pixbuf_loader_set_size(loader,
+                                       backdrop->priv->width,
+                                       backdrop->priv->height);
+            break;
+
+        case XFCE_BACKDROP_IMAGE_SCALED:
+            xscale = (gdouble)backdrop->priv->width / width;
+            yscale = (gdouble)backdrop->priv->height / height;
+            if(xscale < yscale) {
+                yscale = xscale;
+            } else {
+                xscale = yscale;
+            }
+
+            gdk_pixbuf_loader_set_size(loader,
+                                       width * xscale,
+                                       height * yscale);
+            break;
+
+        case XFCE_BACKDROP_IMAGE_ZOOMED:
+        case XFCE_BACKDROP_IMAGE_SPANNING_SCREENS:
+            xscale = (gdouble)backdrop->priv->width / width;
+            yscale = (gdouble)backdrop->priv->height / height;
+            if(xscale < yscale) {
+                xscale = yscale;
+            } else {
+                yscale = xscale;
+            }
+
+            gdk_pixbuf_loader_set_size(loader,
+                                       width * xscale,
+                                       height * yscale);
+            break;
+
+        default:
+            g_critical("Invalid image style: %d\n", (gint)backdrop->priv->image_style);
+    }
+}
+
+static void
+xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
+                               XfceBackdropImageData *image_data)
+{
+    XfceBackdrop *backdrop = image_data->backdrop;
+    GdkPixbuf *final_image, *image, *tmp;
+    gint i, j;
+    gint w, h, iw = 0, ih = 0;
+    XfceBackdropImageStyle istyle;
+    gint dx, dy, xo, yo;
+    gdouble xscale, yscale;
+    GdkInterpType interp;
+
+    TRACE("entering");
+
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    /* canceled? quit now */
+    if(g_cancellable_is_cancelled(image_data->cancellable)) {
+        xfce_backdrop_image_data_release(image_data);
+        return;
+    }
+
+    image = gdk_pixbuf_loader_get_pixbuf(loader);
+    if(image) {
+        iw = gdk_pixbuf_get_width(image);
+        ih = gdk_pixbuf_get_height(image);
+    }
+
+    if(backdrop->priv->width == 0 || backdrop->priv->height == 0) {
+        w = iw;
+        h = ih;
+    } else {
+        w = backdrop->priv->width;
+        h = backdrop->priv->height;
     }
     
-    if(backdrop->priv->image_style == XFCE_BACKDROP_IMAGE_AUTO) {
-        if(ih <= h / 2 && iw <= w / 2)
-            istyle = XFCE_BACKDROP_IMAGE_TILED;
-        else
-            istyle = XFCE_BACKDROP_IMAGE_ZOOMED;
-    } else
-        istyle = backdrop->priv->image_style;
+    istyle = backdrop->priv->image_style;
     
     /* if the image is the same as the screen size, there's no reason to do
      * any scaling at all */
@@ -997,10 +1042,23 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
         else
             interp = GDK_INTERP_BILINEAR;
     }
-    
+
+    final_image = xfce_backdrop_generate_canvas(backdrop);
+
+    /* no image and not canceled? return just the canvas */
+    if(!image && !g_cancellable_is_cancelled(image_data->cancellable)) {
+        DBG("image failed to load, displaying canvas only");
+        backdrop->priv->pix = final_image;
+
+        g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_READY], 0);
+
+        backdrop->priv->image_data = NULL;
+        xfce_backdrop_image_data_release(image_data);
+        return;
+    }
+
     switch(istyle) {
         case XFCE_BACKDROP_IMAGE_CENTERED:
-            image = gdk_pixbuf_new_from_file(backdrop->priv->image_path, NULL);
             dx = MAX((w - iw) / 2, 0);
             dy = MAX((h - ih) / 2, 0);
             xo = MIN((w - iw) / 2, dx);
@@ -1011,13 +1069,13 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
             break;
         
         case XFCE_BACKDROP_IMAGE_TILED:
-            image = gdk_pixbuf_new_from_file(backdrop->priv->image_path, NULL);
             tmp = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, w, h);
             /* Now that the image has been loaded, recalculate the image
              * size because gdk_pixbuf_get_file_info doesn't always return
              * the correct size */
             iw = gdk_pixbuf_get_width(image);
             ih = gdk_pixbuf_get_height(image);
+
             for(i = 0; (i * iw) < w; i++) {
                 for(j = 0; (j * ih) < h; j++) {
                     gint newx = iw * i, newy = ih * j;
@@ -1027,7 +1085,7 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
                         neww = w - newx;
                     if((newy + newh) > h)
                         newh = h - newy;
-                    
+
                     gdk_pixbuf_copy_area(image, 0, 0,
                             neww, newh, tmp, newx, newy);
                 }
@@ -1039,8 +1097,6 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
             break;
         
         case XFCE_BACKDROP_IMAGE_STRETCHED:
-            image = gdk_pixbuf_new_from_file_at_scale(
-                            backdrop->priv->image_path, w, h, FALSE, NULL);
             gdk_pixbuf_composite(image, final_image, 0, 0, w, h,
                     0, 0, 1, 1, interp, 255);
             break;
@@ -1060,15 +1116,13 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
             dx = xo;
             dy = yo;
 
-            image = gdk_pixbuf_new_from_file_at_scale(
-                        backdrop->priv->image_path, iw * xscale,
-                        ih * yscale, TRUE, NULL);
             gdk_pixbuf_composite(image, final_image, dx, dy,
                     iw * xscale, ih * yscale, xo, yo, 1, 1,
                     interp, 255);
             break;
         
         case XFCE_BACKDROP_IMAGE_ZOOMED:
+        case XFCE_BACKDROP_IMAGE_SPANNING_SCREENS:
             xscale = (gdouble)w / iw;
             yscale = (gdouble)h / ih;
             if(xscale < yscale) {
@@ -1081,9 +1135,6 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
                 yo = (h - (ih * yscale)) * 0.5;
             }
 
-            image = gdk_pixbuf_new_from_file_at_scale(
-                                backdrop->priv->image_path, iw * xscale,
-                                ih * yscale, TRUE, NULL);
             gdk_pixbuf_composite(image, final_image, 0, 0,
                     w, h, xo, yo, 1, 1, interp, 255);
             break;
@@ -1091,18 +1142,120 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
         default:
             g_critical("Invalid image style: %d\n", (gint)istyle);
     }
-    
-    if(image)
-        g_object_unref(G_OBJECT(image));
-    
-    if(backdrop->priv->brightness != 0)
-        final_image = adjust_brightness(final_image, backdrop->priv->brightness);
 
-    if(backdrop->priv->saturation > 1.01 || backdrop->priv->saturation < 0.99) {
-        gdk_pixbuf_saturate_and_pixelate(final_image, final_image,
-                                         (gfloat)backdrop->priv->saturation,
-                                         FALSE);
+    /* keep the backdrop and emit the signal if it hasn't been canceled */
+    if(!g_cancellable_is_cancelled(image_data->cancellable)) {
+        backdrop->priv->pix = final_image;
+        g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_READY], 0);
     }
-    
-    return final_image;
+
+    backdrop->priv->image_data = NULL;
+    xfce_backdrop_image_data_release(image_data);
+}
+
+static void
+xfce_backdrop_file_ready_cb(GObject *source_object,
+                            GAsyncResult *res,
+                            gpointer user_data)
+{
+    GFile *file = G_FILE(source_object);
+    XfceBackdropImageData *image_data = user_data;
+    GFileInputStream *input_stream = g_file_read_finish(file, res, NULL);
+
+    TRACE("entering");
+
+    /* If this fails then close the loader, it will only display the selected
+     * backdrop color */
+    if(input_stream == NULL) {
+        gdk_pixbuf_loader_close(image_data->loader, NULL);
+        return;
+    }
+
+    g_input_stream_read_async(G_INPUT_STREAM(input_stream),
+                              image_data->image_buffer,
+                              XFCE_BACKDROP_BUFFER_SIZE,
+                              G_PRIORITY_LOW,
+                              image_data->cancellable,
+                              xfce_backdrop_file_input_stream_ready_cb,
+                              image_data);
+}
+
+static void
+xfce_backdrop_file_input_stream_ready_cb(GObject *source_object,
+                                         GAsyncResult *res,
+                                         gpointer user_data)
+{
+    GInputStream *stream = G_INPUT_STREAM(source_object);
+    XfceBackdropImageData *image_data = user_data;
+    gssize bytes = g_input_stream_read_finish(stream, res, NULL);
+
+    if(bytes == -1 || bytes == 0) {
+        /* If there was an error reading the stream or it completed, clean
+         * up and close the pixbuf loader (which will handle both conditions) */
+        g_input_stream_close(stream, image_data->cancellable, NULL);
+        g_object_unref(source_object);
+
+        gdk_pixbuf_loader_close(image_data->loader, NULL);
+        return;
+    }
+
+    if(gdk_pixbuf_loader_write(image_data->loader, image_data->image_buffer, bytes, NULL)) {
+        g_input_stream_read_async(stream,
+                                  image_data->image_buffer,
+                                  XFCE_BACKDROP_BUFFER_SIZE,
+                                  G_PRIORITY_LOW,
+                                  image_data->cancellable,
+                                  xfce_backdrop_file_input_stream_ready_cb,
+                                  image_data);
+    } else {
+        /* If we got here, the loader will be closed, and will not accept
+         * further writes. */
+        g_input_stream_close(stream, image_data->cancellable, NULL);
+        g_object_unref(source_object);
+    }
+}
+
+/* returns TRUE if they have identical settings. */
+gboolean
+xfce_backdrop_compare_backdrops(XfceBackdrop *backdrop_a,
+                                XfceBackdrop *backdrop_b)
+{
+    if(g_strcmp0(backdrop_a->priv->image_path, backdrop_b->priv->image_path) != 0) {
+        DBG("filename different");
+        return FALSE;
+    }
+
+    if(backdrop_a->priv->image_style != backdrop_b->priv->image_style) {
+        DBG("image_style different");
+        return FALSE;
+    }
+
+    if(backdrop_a->priv->color_style != backdrop_b->priv->color_style) {
+        DBG("color_style different");
+        return FALSE;
+    }
+
+    /* Every color style uses color1 except for transparent which does not need
+     * a color check */
+    if(backdrop_a->priv->color_style != XFCE_BACKDROP_COLOR_TRANSPARENT &&
+       !gdk_color_equal(&backdrop_a->priv->color1, &backdrop_b->priv->color1)) {
+        DBG("colors different");
+        return FALSE;
+    }
+
+    /* When the style is set to gradient then we should check color2 as well */
+    if((backdrop_a->priv->color_style == XFCE_BACKDROP_COLOR_HORIZ_GRADIENT ||
+        backdrop_a->priv->color_style == XFCE_BACKDROP_COLOR_VERT_GRADIENT) &&
+       !gdk_color_equal(&backdrop_a->priv->color2, &backdrop_b->priv->color2)) {
+        DBG("colors different");
+        return FALSE;
+    }
+
+    if(backdrop_a->priv->cycle_backdrop != backdrop_b->priv->cycle_backdrop ||
+       backdrop_a->priv->cycle_backdrop == TRUE) {
+        DBG("backdrop cycle different");
+        return FALSE;
+    }
+
+    return TRUE;
 }
