@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifndef _XFCE_BACKDROP_H_
@@ -40,12 +40,13 @@ typedef struct _XfceBackdropPriv XfceBackdropPriv;
 
 typedef enum
 {
-    XFCE_BACKDROP_IMAGE_AUTO = 0,
+    XFCE_BACKDROP_IMAGE_NONE = 0,
     XFCE_BACKDROP_IMAGE_CENTERED,
     XFCE_BACKDROP_IMAGE_TILED,
     XFCE_BACKDROP_IMAGE_STRETCHED,
     XFCE_BACKDROP_IMAGE_SCALED,
     XFCE_BACKDROP_IMAGE_ZOOMED,
+    XFCE_BACKDROP_IMAGE_SPANNING_SCREENS,
 } XfceBackdropImageStyle;
 
 typedef enum
@@ -70,6 +71,8 @@ struct _XfceBackdropClass
     
     /*< signals >*/
     void (*changed)(XfceBackdrop *backdrop);
+    void (*cycle)(XfceBackdrop *backdrop);
+    void (*ready)(XfceBackdrop *backdrop);
 };
 
 GType xfce_backdrop_get_type             (void) G_GNUC_CONST;
@@ -99,10 +102,6 @@ void xfce_backdrop_set_second_color      (XfceBackdrop *backdrop,
 void xfce_backdrop_get_second_color      (XfceBackdrop *backdrop,
                                           GdkColor *color);
 
-void xfce_backdrop_set_show_image        (XfceBackdrop *backdrop,
-                                          gboolean show_image);
-gboolean xfce_backdrop_get_show_image    (XfceBackdrop *backdrop);
-
 void xfce_backdrop_set_image_style       (XfceBackdrop *backdrop,
                                           XfceBackdropImageStyle style);
 XfceBackdropImageStyle xfce_backdrop_get_image_style
@@ -113,16 +112,25 @@ void xfce_backdrop_set_image_filename    (XfceBackdrop *backdrop,
 G_CONST_RETURN gchar *xfce_backdrop_get_image_filename
                                          (XfceBackdrop *backdrop);
 
-void xfce_backdrop_set_brightness        (XfceBackdrop *backdrop,
-                                          gint brightness);
-gint xfce_backdrop_get_brightness        (XfceBackdrop *backdrop);
+void xfce_backdrop_set_cycle_backdrop    (XfceBackdrop *backdrop,
+                                          gboolean cycle_backdrop);
+gboolean xfce_backdrop_get_cycle_backdrop(XfceBackdrop *backdrop);
 
-void xfce_backdrop_set_saturation        (XfceBackdrop *backdrop,
-                                          gdouble saturation);
-gdouble xfce_backdrop_get_saturation     (XfceBackdrop *backdrop);
+void xfce_backdrop_set_cycle_timer       (XfceBackdrop *backdrop,
+                                          guint cycle_timer);
+guint xfce_backdrop_get_cycle_timer      (XfceBackdrop *backdrop);
+
+void xfce_backdrop_set_random_order      (XfceBackdrop *backdrop,
+                                          gboolean random_order);
+gboolean xfce_backdrop_get_random_order  (XfceBackdrop *backdrop);
 
 GdkPixbuf *xfce_backdrop_get_pixbuf      (XfceBackdrop *backdrop);
 
+void xfce_backdrop_generate_async        (XfceBackdrop *backdrop);
+
+gboolean xfce_backdrop_compare_backdrops (XfceBackdrop *backdrop_a,
+                                          XfceBackdrop *backdrop_b);
+
 G_END_DECLS
 
 #endif
