@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
- * Copied, renamed, and hacked to pieces by Brian Tarricone <brian@tarricone.org>.
+ * Copied, renamed, and hacked to pieces by Brian Tarricone <bjt23@cornell.edu>.
  * Original code from Thunar.
  */
 
@@ -59,17 +59,14 @@ enum
 
 enum
 {
-  TARGET_TEXT_URI_LIST,
   TARGET_GNOME_COPIED_FILES,
   TARGET_UTF8_STRING,
 };
 
 
 
-static void xfdesktop_clipboard_manager_class_init        (gpointer                        g_class,
-                                                           gpointer                        class_data);
-static void xfdesktop_clipboard_manager_init              (GTypeInstance                  *instance,
-                                                           gpointer                        g_class);
+static void xfdesktop_clipboard_manager_class_init        (XfdesktopClipboardManagerClass *klass);
+static void xfdesktop_clipboard_manager_init              (XfdesktopClipboardManager      *manager);
 static void xfdesktop_clipboard_manager_finalize          (GObject                        *object);
 static void xfdesktop_clipboard_manager_get_property      (GObject                        *object,
                                                            guint                           prop_id,
@@ -81,11 +78,11 @@ static void xfdesktop_clipboard_manager_file_destroyed    (XfdesktopClipboardMan
 static void xfdesktop_clipboard_manager_owner_changed     (GtkClipboard                   *clipboard,
                                                            GdkEventOwnerChange            *event,
                                                            XfdesktopClipboardManager      *manager);
-
+#if 0
 static void xfdesktop_clipboard_manager_contents_received (GtkClipboard                   *clipboard,
                                                            GtkSelectionData               *selection_data,
                                                            gpointer                        user_data);
-
+#endif
 static void xfdesktop_clipboard_manager_targets_received  (GtkClipboard                   *clipboard,
                                                            GtkSelectionData               *selection_data,
                                                            gpointer                        user_data);
@@ -133,7 +130,6 @@ typedef struct
 
 static const GtkTargetEntry clipboard_targets[] =
 {
-  { "text/uri-list", 0, TARGET_TEXT_URI_LIST },
   { "x-special/gnome-copied-files", 0, TARGET_GNOME_COPIED_FILES },
   { "UTF8_STRING", 0, TARGET_UTF8_STRING }
 };
@@ -156,12 +152,12 @@ xfdesktop_clipboard_manager_get_type (void)
         sizeof (XfdesktopClipboardManagerClass),
         NULL,
         NULL,
-        xfdesktop_clipboard_manager_class_init,
+        (GClassInitFunc) xfdesktop_clipboard_manager_class_init,
         NULL,
         NULL,
         sizeof (XfdesktopClipboardManager),
         0,
-        xfdesktop_clipboard_manager_init,
+        (GInstanceInitFunc) xfdesktop_clipboard_manager_init,
         NULL,
       };
 
@@ -174,15 +170,14 @@ xfdesktop_clipboard_manager_get_type (void)
 
 
 static void
-xfdesktop_clipboard_manager_class_init (gpointer g_class,
-                                        gpointer class_data)
+xfdesktop_clipboard_manager_class_init (XfdesktopClipboardManagerClass *klass)
 {
   GObjectClass *gobject_class;
 
   /* determine the parent type class */
-  xfdesktop_clipboard_manager_parent_class = g_type_class_peek_parent (g_class);
+  xfdesktop_clipboard_manager_parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class = G_OBJECT_CLASS (g_class);
+  gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = xfdesktop_clipboard_manager_finalize;
   gobject_class->get_property = xfdesktop_clipboard_manager_get_property;
 
@@ -193,7 +188,7 @@ xfdesktop_clipboard_manager_class_init (gpointer g_class,
    * this #XfdesktopClipboardManager can be pasted into the desktop
    * displayed by #XfdesktopIconView.
    **/
-  g_object_class_install_property (g_class,
+  g_object_class_install_property (gobject_class,
                                    PROP_CAN_PASTE,
                                    g_param_spec_boolean ("can-paste", "can-pase", "can-paste",
                                                          FALSE,
@@ -207,7 +202,7 @@ xfdesktop_clipboard_manager_class_init (gpointer g_class,
    **/
   manager_signals[CHANGED] =
     g_signal_new (I_("changed"),
-                  G_TYPE_FROM_CLASS (g_class),
+                  G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (XfdesktopClipboardManagerClass, changed),
                   NULL, NULL,
@@ -218,11 +213,8 @@ xfdesktop_clipboard_manager_class_init (gpointer g_class,
 
 
 static void
-xfdesktop_clipboard_manager_init (GTypeInstance *instance,
-                                  gpointer       g_class)
+xfdesktop_clipboard_manager_init (XfdesktopClipboardManager *manager)
 {
-  XfdesktopClipboardManager *manager = XFDESKTOP_CLIPBOARD_MANAGER (instance);
-
   manager->x_special_gnome_copied_files = gdk_atom_intern ("x-special/gnome-copied-files", FALSE);
 }
 
@@ -430,7 +422,7 @@ xfdesktop_clipboard_manager_targets_received (GtkClipboard     *clipboard,
   GdkAtom                *targets;
   gint                    n_targets;
   gint                    n;
-
+  
   g_return_if_fail (GTK_IS_CLIPBOARD (clipboard));
   g_return_if_fail (XFDESKTOP_IS_CLIPBOARD_MANAGER (manager));
   g_return_if_fail (manager->clipboard == clipboard);
@@ -450,7 +442,7 @@ xfdesktop_clipboard_manager_targets_received (GtkClipboard     *clipboard,
 
       g_free (targets);
     }
-
+  
   /* notify listeners that we have a new clipboard state */
   g_signal_emit (G_OBJECT (manager), manager_signals[CHANGED], 0);
   g_object_notify (G_OBJECT (manager), "can-paste");
@@ -467,43 +459,39 @@ xfdesktop_clipboard_manager_get_callback (GtkClipboard     *clipboard,
                                           gpointer          user_data)
 {
   XfdesktopClipboardManager *manager = XFDESKTOP_CLIPBOARD_MANAGER (user_data);
+  GList                  *file_list = NULL;
+  gchar                  *string_list;
+  gchar                  *data;
 
   g_return_if_fail (GTK_IS_CLIPBOARD (clipboard));
   g_return_if_fail (XFDESKTOP_IS_CLIPBOARD_MANAGER (manager));
   g_return_if_fail (manager->clipboard == clipboard);
 
   /* determine the file list from the icon list */
-  GList *file_list = xfdesktop_file_utils_file_icon_list_to_file_list (manager->files);
+  file_list = xfdesktop_file_utils_file_icon_list_to_file_list (manager->files);
+
+  /* determine the string representation of the file list */
+  string_list = xfdesktop_file_utils_file_list_to_string (file_list);
 
   switch (target_info)
     {
-    case TARGET_TEXT_URI_LIST: {
-      gchar **uris = xfdesktop_file_utils_file_list_to_uri_array (file_list);
-      gtk_selection_data_set_uris (selection_data, uris);
-      g_strfreev (uris);
-      break;
-    }
-
-    case TARGET_GNOME_COPIED_FILES: {
-      const gchar *prefix = manager->files_cutted ? "cut\n" : "copy\n";
-      gsize len = 0;
-      gchar *str = xfdesktop_file_utils_file_list_to_string(file_list, prefix, FALSE, &len);
+    case TARGET_GNOME_COPIED_FILES:
+      data = g_strconcat (manager->files_cutted ? "cut\n" : "copy\n", string_list, NULL);
       gtk_selection_data_set (selection_data,
                               gtk_selection_data_get_target(selection_data),
                               8,
-                              (guchar *) str,
-                              len);
-      g_free (str);
+                              (guchar *) data,
+                              strlen (data));
+      g_free (data);
       break;
-    }
 
-    case TARGET_UTF8_STRING: {
-      gsize len = 0;
-      gchar *str = xfdesktop_file_utils_file_list_to_string(file_list, NULL, TRUE, &len);
-      gtk_selection_data_set_text(selection_data, str, len);
-      g_free(str);
+    case TARGET_UTF8_STRING:
+      gtk_selection_data_set (selection_data,
+                              gtk_selection_data_get_target(selection_data),
+                              8,
+                              (guchar *) string_list,
+                              strlen (string_list));
       break;
-    }
 
     default:
       g_assert_not_reached ();
@@ -511,6 +499,7 @@ xfdesktop_clipboard_manager_get_callback (GtkClipboard     *clipboard,
 
   /* cleanup */
   xfdesktop_file_utils_file_list_free (file_list);
+  g_free (string_list);
 }
 
 
@@ -564,9 +553,9 @@ xfdesktop_clipboard_manager_transfer_files (XfdesktopClipboardManager *manager,
   /* setup the new file list */
   for (lp = files, manager->files = NULL; lp != NULL; lp = lp->next)
     {
-      file = XFDESKTOP_FILE_ICON(g_object_ref (G_OBJECT (lp->data)));
+      file = g_object_ref (G_OBJECT (lp->data));
       manager->files = g_list_prepend (manager->files, file);
-      g_object_weak_ref(G_OBJECT(file),
+      g_object_weak_ref(G_OBJECT(file), 
                         (GWeakNotify)xfdesktop_clipboard_manager_file_destroyed,
                         manager);
     }
@@ -624,7 +613,7 @@ xfdesktop_clipboard_manager_get_for_display (GdkDisplay *display)
 
   /* allocate a new manager */
   manager = g_object_new (XFDESKTOP_TYPE_CLIPBOARD_MANAGER, NULL);
-  manager->clipboard = GTK_CLIPBOARD (g_object_ref (G_OBJECT (clipboard)));
+  manager->clipboard = g_object_ref (G_OBJECT (clipboard));
   g_object_set_qdata (G_OBJECT (clipboard), xfdesktop_clipboard_manager_quark, manager);
 
   /* listen for the "owner-change" signal on the clipboard */
@@ -650,7 +639,7 @@ xfdesktop_clipboard_manager_has_cutted_file (XfdesktopClipboardManager *manager,
                                              const XfdesktopFileIcon       *file)
 {
   g_return_val_if_fail (XFDESKTOP_IS_CLIPBOARD_MANAGER (manager), FALSE);
-  g_return_val_if_fail (XFDESKTOP_IS_FILE_ICON((gpointer)file), FALSE);
+  g_return_val_if_fail (XFDESKTOP_IS_FILE_ICON (file), FALSE);
 
   return (manager->files_cutted && g_list_find (manager->files, file) != NULL);
 }
@@ -732,7 +721,7 @@ xfdesktop_clipboard_manager_paste_files (XfdesktopClipboardManager *manager,
 
   /* prepare the paste request */
   request = g_slice_new0 (XfdesktopClipboardPasteRequest);
-  request->manager = XFDESKTOP_CLIPBOARD_MANAGER (g_object_ref (G_OBJECT (manager)));
+  request->manager = g_object_ref (G_OBJECT (manager));
   request->target_file = g_object_ref (target_file);
   request->widget = widget;
 
