@@ -15,8 +15,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -63,6 +63,8 @@
 #include "xfdesktop-file-utils.h"
 #include "xfdesktop-trash-proxy.h"
 
+static void xfdesktop_file_utils_add_emblems(GdkPixbuf *pix, GList *emblems);
+
 gboolean
 xfdesktop_file_utils_is_desktop_file(GFileInfo *info)
 {
@@ -322,7 +324,7 @@ xfdesktop_file_utils_mount_is_internal (GMount *mount)
 gboolean
 xfdesktop_file_utils_volume_is_removable(GVolume *volume)
 {
-  gboolean can_eject = FALSE;
+  gboolean can_eject;
   gboolean can_mount = FALSE;
   gboolean can_unmount = FALSE;
   gboolean is_removable = FALSE;
@@ -375,7 +377,8 @@ xfdesktop_file_utils_file_icon_list_to_file_list(GList *icon_list)
     for(l = icon_list; l; l = l->next) {
         icon = XFDESKTOP_FILE_ICON(l->data);
         file = xfdesktop_file_icon_peek_file(icon);
-        file_list = g_list_prepend(file_list, g_object_ref(file));
+        if(file)
+            file_list = g_list_prepend(file_list, g_object_ref(file));
     }
 
     return g_list_reverse(file_list);
@@ -488,42 +491,54 @@ xfdesktop_file_utils_get_fallback_icon(gint size)
 }
 
 GdkPixbuf *
-xfdesktop_file_utils_get_icon(const gchar *custom_icon_name,
-                              GIcon *icon,
-                              gint size,
-                              const GdkPixbuf *emblem,
+xfdesktop_file_utils_get_icon(GIcon *icon,
+                              gint width,
+                              gint height,
                               guint opacity)
 {
     GtkIconTheme *itheme = gtk_icon_theme_get_default();
     GdkPixbuf *pix_theme = NULL, *pix = NULL;
+    GIcon *base_icon = NULL;
+    gint size = MIN(width, height);
 
-    if(custom_icon_name) {
-        pix_theme = gtk_icon_theme_load_icon(itheme, custom_icon_name, size,
-                                             ITHEME_FLAGS, NULL);
-        if(!pix_theme && *custom_icon_name == '/' && g_file_test(custom_icon_name, G_FILE_TEST_IS_REGULAR))
-            pix_theme = gdk_pixbuf_new_from_file_at_size(custom_icon_name, size, size, NULL);
-    }
+    g_return_val_if_fail(width > 0 && height > 0 && icon != NULL, NULL);
 
-    if(!pix_theme && icon) {
-        if(G_IS_THEMED_ICON(icon)) {
-          GtkIconInfo *icon_info = gtk_icon_theme_lookup_by_gicon(itheme,
-                                                                  icon, size,
-                                                                  ITHEME_FLAGS);
-          if(icon_info) {
-              pix_theme = gtk_icon_info_load_icon(icon_info, NULL);
-              gtk_icon_info_free(icon_info);
-          }
-        } else if(G_IS_LOADABLE_ICON(icon)) {
-            GInputStream *stream = g_loadable_icon_load(G_LOADABLE_ICON(icon),
-                                                        size, NULL, NULL, NULL);
-            if(stream) {
-                pix = gdk_pixbuf_new_from_stream(stream, NULL, NULL);
-                g_object_unref(stream);
-            }
+    /* Extract the base icon if available */
+    if(G_IS_EMBLEMED_ICON(icon))
+        base_icon = g_emblemed_icon_get_icon(G_EMBLEMED_ICON(icon));
+    else
+        base_icon = icon;
+
+    if(!base_icon)
+        return NULL;
+
+    if(G_IS_THEMED_ICON(base_icon)) {
+      GtkIconInfo *icon_info = gtk_icon_theme_lookup_by_gicon(itheme,
+                                                              base_icon, size,
+                                                              ITHEME_FLAGS);
+      if(icon_info) {
+          pix_theme = gtk_icon_info_load_icon(icon_info, NULL);
+          gtk_icon_info_free(icon_info);
+      }
+    } else if(G_IS_LOADABLE_ICON(base_icon)) {
+        GInputStream *stream = g_loadable_icon_load(G_LOADABLE_ICON(base_icon),
+                                                    size, NULL, NULL, NULL);
+        if(stream) {
+            pix = gdk_pixbuf_new_from_stream_at_scale(stream, width, height, TRUE, NULL, NULL);
+            g_object_unref(stream);
         }
+    } else if(G_IS_FILE_ICON(base_icon)) {
+        GFile *file = g_file_icon_get_file(G_FILE_ICON(icon));
+        gchar *path = g_file_get_path(file);
+
+        pix = gdk_pixbuf_new_from_file_at_size(path, width, height, NULL);
+
+        g_free(path);
+        g_object_unref(file);
     }
 
-    if(G_LIKELY(pix_theme)) {
+
+    if(pix_theme) {
         /* we can't edit thsese icons */
         pix = gdk_pixbuf_copy(pix_theme);
         g_object_unref(G_OBJECT(pix_theme));
@@ -540,35 +555,9 @@ xfdesktop_file_utils_get_icon(const gchar *custom_icon_name,
         return NULL;
     }
 
-    if(emblem) {
-        gint emblem_pix_size = gdk_pixbuf_get_width(emblem);
-        gint dest_size = size - emblem_pix_size;
-
-        /* if we're using the fallback icon, we don't want to draw an emblem on
-         * it, since other icons might use it without the emblem */
-        if(G_UNLIKELY(pix == xfdesktop_fallback_icon)) {
-            GdkPixbuf *tmp = gdk_pixbuf_copy(pix);
-            g_object_unref(G_OBJECT(pix));
-            pix = tmp;
-        }
-
-        if(dest_size < 0)
-            g_critical("xfdesktop_file_utils_get_file_icon(): (dest_size > 0) failed");
-        else {
-            DBG("calling gdk_pixbuf_composite(%p, %p, %d, %d, %d, %d, %.1f, %.1f, %.1f, %.1f, %d, %d)",
-                emblem, pix,
-                dest_size, dest_size,
-                emblem_pix_size, emblem_pix_size,
-                (gdouble)dest_size, (gdouble)dest_size,
-                1.0, 1.0, GDK_INTERP_BILINEAR, 255);
-
-            gdk_pixbuf_composite(emblem, pix,
-                                 dest_size, dest_size,
-                                 emblem_pix_size, emblem_pix_size,
-                                 dest_size, dest_size,
-                                 1.0, 1.0, GDK_INTERP_BILINEAR, 255);
-        }
-    }
+    /* Add the emblems */
+    if(G_IS_EMBLEMED_ICON(icon))
+        xfdesktop_file_utils_add_emblems(pix, g_emblemed_icon_get_emblems(G_EMBLEMED_ICON(icon)));
 
     if(opacity != 100) {
         GdkPixbuf *tmp = exo_gdk_pixbuf_lucent(pix, opacity);
@@ -579,18 +568,109 @@ xfdesktop_file_utils_get_icon(const gchar *custom_icon_name,
     return pix;
 }
 
+static void
+xfdesktop_file_utils_add_emblems(GdkPixbuf *pix, GList *emblems)
+{
+    GdkPixbuf *emblem_pix;
+    gint max_emblems;
+    gint pix_width, pix_height;
+    gint emblem_size;
+    gint dest_x, dest_y, dest_width, dest_height;
+    gint position;
+    GList *iter;
+    GtkIconTheme *itheme = gtk_icon_theme_get_default();
+
+    g_return_if_fail(pix != NULL);
+
+    pix_width = gdk_pixbuf_get_width(pix);
+    pix_height = gdk_pixbuf_get_height(pix);
+
+    emblem_size = MIN(pix_width, pix_height) / 2;
+
+    /* render up to four emblems for sizes from 48 onwards, else up to 2 emblems */
+    max_emblems = (pix_height < 48 && pix_width < 48) ? 2 : 4;
+
+    for(iter = emblems, position = 0; iter != NULL && position < max_emblems; iter = iter->next) {
+        /* extract the icon from the emblem and load it */
+        GIcon *emblem = g_emblem_get_icon(iter->data);
+        GtkIconInfo *icon_info = gtk_icon_theme_lookup_by_gicon(itheme,
+                                                                emblem,
+                                                                emblem_size,
+                                                                ITHEME_FLAGS);
+        if(icon_info) {
+            emblem_pix = gtk_icon_info_load_icon(icon_info, NULL);
+            gtk_icon_info_free(icon_info);
+        }
+
+        if(emblem_pix) {
+            if(gdk_pixbuf_get_width(emblem_pix) != emblem_size
+               || gdk_pixbuf_get_height(emblem_pix) != emblem_size)
+            {
+                GdkPixbuf *tmp = gdk_pixbuf_scale_simple(emblem_pix,
+                                                         emblem_size,
+                                                         emblem_size,
+                                                         GDK_INTERP_BILINEAR);
+                g_object_unref(emblem_pix);
+                emblem_pix = tmp;
+            }
+
+            dest_width = pix_width - emblem_size;
+            dest_height = pix_height - emblem_size;
+
+            switch(position) {
+                case 0: /* bottom right */
+                    dest_x = dest_width;
+                    dest_y = dest_height;
+                    break;
+                case 1: /* bottom left */
+                    dest_x = 0;
+                    dest_y = dest_height;
+                    break;
+                case 2: /* upper right */
+                    dest_x = dest_width;
+                    dest_y = 0;
+                    break;
+                case 3: /* upper left */
+                    dest_x = dest_y = 0;
+                    break;
+                default:
+                    g_warning("Invalid emblem position in xfdesktop_file_utils_add_emblems");
+            }
+
+            DBG("calling gdk_pixbuf_composite(%p, %p, %d, %d, %d, %d, %d, %d, %.1f, %.1f, %d, %d) pixbuf w: %d h: %d",
+                emblem_pix, pix,
+                dest_x, dest_y,
+                emblem_size, emblem_size,
+                dest_x, dest_y,
+                1.0, 1.0, GDK_INTERP_BILINEAR, 255, pix_width, pix_height);
+
+            /* Add the emblem */
+            gdk_pixbuf_composite(emblem_pix, pix,
+                                 dest_x, dest_y,
+                                 emblem_size, emblem_size,
+                                 dest_x, dest_y,
+                                 1.0, 1.0, GDK_INTERP_BILINEAR, 255);
+
+            g_object_unref(emblem_pix);
+            emblem_pix = NULL;
+
+            position++;
+        }
+    }
+}
+
 void
 xfdesktop_file_utils_set_window_cursor(GtkWindow *window,
                                        GdkCursorType cursor_type)
 {
     GdkCursor *cursor;
 
-    if(!window || !GTK_WIDGET(window)->window)
+    if(!window || !gtk_widget_get_window(GTK_WIDGET(window)))
         return;
 
     cursor = gdk_cursor_new(cursor_type);
     if(G_LIKELY(cursor)) {
-        gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
+        gdk_window_set_cursor(gtk_widget_get_window(GTK_WIDGET(window)), cursor);
         gdk_cursor_unref(cursor);
     }
 }
@@ -715,6 +795,21 @@ xfdesktop_file_utils_open_folder(GFile *file,
     g_free(uri);
 }
 
+static void
+xfdesktop_file_utils_async_cb(DBusGProxy *proxy, GError *error, gpointer userdata)
+{
+    GtkWindow *parent = GTK_WINDOW(userdata);
+
+    if(error != NULL) {
+        xfce_message_dialog(parent,
+                            _("Error"), GTK_STOCK_DIALOG_ERROR,
+                            _("The requested operation could not be completed"),
+                            error->message, GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
+                            NULL);
+        g_error_free(error);
+    }
+}
+
 void
 xfdesktop_file_utils_rename_file(GFile *file,
                                  GdkScreen *screen,
@@ -730,25 +825,16 @@ xfdesktop_file_utils_rename_file(GFile *file,
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
     if(fileman_proxy) {
-        GError *error = NULL;
         gchar *uri = g_file_get_uri(file);
         gchar *display_name = gdk_screen_make_display_name(screen);
         gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        if(!xfdesktop_file_manager_proxy_rename_file(fileman_proxy,
-                                                     uri, display_name, startup_id,
-                                                     &error))
-        {
-            xfce_message_dialog(parent,
-                                _("Rename Error"), GTK_STOCK_DIALOG_ERROR,
-                                _("The file could not be renamed"),
-                                error->message, GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_error_free(error);
-        }
+        xfdesktop_file_manager_proxy_rename_file_async(fileman_proxy,
+                                                       uri, display_name, startup_id,
+                                                       (xfdesktop_file_manager_proxy_rename_file_reply)xfdesktop_file_utils_async_cb,
+                                                       parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
@@ -780,7 +866,6 @@ xfdesktop_file_utils_unlink_files(GList *files,
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
     if(fileman_proxy) {
-        GError *error = NULL;
         guint nfiles = g_list_length(files);
         gchar **uris = g_new0(gchar *, nfiles+1);
         gchar *display_name = gdk_screen_make_display_name(screen);
@@ -795,19 +880,11 @@ xfdesktop_file_utils_unlink_files(GList *files,
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        if(!xfdesktop_file_manager_proxy_unlink_files(fileman_proxy,
-                                                      NULL, (const gchar **)uris,
-                                                      display_name, startup_id,
-                                                      &error))
-        {
-            xfce_message_dialog(parent,
-                                _("Delete Error"), GTK_STOCK_DIALOG_ERROR,
-                                _("The selected files could not be deleted"),
-                                error->message, GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_error_free(error);
-        }
+        xfdesktop_file_manager_proxy_unlink_files_async(fileman_proxy,
+                                                        NULL, (const gchar **)uris,
+                                                        display_name, startup_id,
+                                                        (xfdesktop_file_manager_proxy_unlink_files_reply)xfdesktop_file_utils_async_cb,
+                                                        parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
@@ -839,7 +916,6 @@ xfdesktop_file_utils_trash_files(GList *files,
 
     trash_proxy = xfdesktop_file_utils_peek_trash_proxy();
     if(trash_proxy) {
-        GError *error = NULL;
         guint nfiles = g_list_length(files);
         gchar **uris = g_new0(gchar *, nfiles+1);
         gchar *display_name = gdk_screen_make_display_name(screen);
@@ -854,19 +930,11 @@ xfdesktop_file_utils_trash_files(GList *files,
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        if(!xfdesktop_trash_proxy_move_to_trash(trash_proxy,
-                                                (const gchar **)uris,
-                                                display_name, startup_id,
-                                                &error))
-        {
-            xfce_message_dialog(parent,
-                                _("Trash Error"), GTK_STOCK_DIALOG_ERROR,
-                                _("The selected files could not be moved to the trash"),
-                                error->message, GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_error_free(error);
-        }
+        xfdesktop_trash_proxy_move_to_trash_async(trash_proxy,
+                                                  (const gchar **)uris,
+                                                  display_name, startup_id,
+                                                  (xfdesktop_trash_proxy_move_to_trash_reply)xfdesktop_file_utils_async_cb,
+                                                  parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
@@ -896,24 +964,15 @@ xfdesktop_file_utils_empty_trash(GdkScreen *screen,
 
     trash_proxy = xfdesktop_file_utils_peek_trash_proxy();
     if(trash_proxy) {
-        GError *error = NULL;
         gchar *display_name = gdk_screen_make_display_name(screen);
         gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        if(!xfdesktop_trash_proxy_empty_trash(trash_proxy,
-                                              display_name, startup_id,
-                                              &error))
-        {
-            xfce_message_dialog(parent,
-                                _("Trash Error"), GTK_STOCK_DIALOG_ERROR,
-                                _("Could not empty the trash"),
-                                error->message, GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_error_free(error);
-        }
+        xfdesktop_trash_proxy_empty_trash_async(trash_proxy,
+                                                display_name, startup_id,
+                                                (xfdesktop_trash_proxy_empty_trash_reply)xfdesktop_file_utils_async_cb,
+                                                parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
@@ -945,27 +1004,18 @@ xfdesktop_file_utils_create_file(GFile *parent_folder,
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
     if(fileman_proxy) {
-        GError *error = NULL;
         gchar *parent_directory = g_file_get_uri(parent_folder);
         gchar *display_name = gdk_screen_make_display_name(screen);
         gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        if(!xfdesktop_file_manager_proxy_create_file(fileman_proxy,
-                                                     parent_directory,
-                                                     content_type, display_name,
-                                                     startup_id,
-                                                     &error))
-        {
-            xfce_message_dialog(parent,
-                                _("Create File Error"), GTK_STOCK_DIALOG_ERROR,
-                                _("Could not create a new file"),
-                                error->message, GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_error_free(error);
-        }
+        xfdesktop_file_manager_proxy_create_file_async(fileman_proxy,
+                                                       parent_directory,
+                                                       content_type, display_name,
+                                                       startup_id,
+                                                       (xfdesktop_file_manager_proxy_create_file_reply)xfdesktop_file_utils_async_cb,
+                                                       parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
@@ -999,7 +1049,6 @@ xfdesktop_file_utils_create_file_from_template(GFile *parent_folder,
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
     if(fileman_proxy) {
-        GError *error = NULL;
         gchar *parent_directory = g_file_get_uri(parent_folder);
         gchar *template_uri = g_file_get_uri(template_file);
         gchar *display_name = gdk_screen_make_display_name(screen);
@@ -1007,21 +1056,13 @@ xfdesktop_file_utils_create_file_from_template(GFile *parent_folder,
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        if(!xfdesktop_file_manager_proxy_create_file_from_template(fileman_proxy,
-                                                                   parent_directory,
-                                                                   template_uri,
-                                                                   display_name,
-                                                                   startup_id,
-                                                                   &error))
-        {
-            xfce_message_dialog(parent,
-                                _("Create Document Error"), GTK_STOCK_DIALOG_ERROR,
-                                _("Could not create a new document from the template"),
-                                error->message, GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_error_free(error);
-        }
+        xfdesktop_file_manager_proxy_create_file_from_template_async(fileman_proxy,
+                                                                     parent_directory,
+                                                                     template_uri,
+                                                                     display_name,
+                                                                     startup_id,
+                                                                     (xfdesktop_file_manager_proxy_create_file_from_template_reply)xfdesktop_file_utils_async_cb,
+                                                                     parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
@@ -1053,25 +1094,16 @@ xfdesktop_file_utils_show_properties_dialog(GFile *file,
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
     if(fileman_proxy) {
-        GError *error = NULL;
         gchar *uri = g_file_get_uri(file);
         gchar *display_name = gdk_screen_make_display_name(screen);
         gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        if(!xfdesktop_file_manager_proxy_display_file_properties(fileman_proxy,
-                                                                 uri, display_name, startup_id,
-                                                                 &error))
-        {
-            xfce_message_dialog(parent,
-                                _("File Properties Error"), GTK_STOCK_DIALOG_ERROR,
-                                _("The file properties dialog could not be opened"),
-                                error->message, GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_error_free(error);
-        }
+        xfdesktop_file_manager_proxy_display_file_properties_async(fileman_proxy,
+                                                                   uri, display_name, startup_id,
+                                                                   (xfdesktop_file_manager_proxy_display_file_properties_reply)xfdesktop_file_utils_async_cb,
+                                                                   parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
@@ -1103,25 +1135,16 @@ xfdesktop_file_utils_launch(GFile *file,
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
     if(fileman_proxy) {
-        GError *error = NULL;
         gchar *uri = g_file_get_uri(file);
         gchar *display_name = gdk_screen_make_display_name(screen);
         gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        if(!xfdesktop_file_manager_proxy_launch(fileman_proxy,
-                                                uri, display_name, startup_id,
-                                                &error))
-        {
-            xfce_message_dialog(parent,
-                                _("Launch Error"), GTK_STOCK_DIALOG_ERROR,
-                                _("The file could not be opened"),
-                                error->message,
-                                GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
-
-            g_error_free(error);
-        }
+        xfdesktop_file_manager_proxy_launch_async(fileman_proxy,
+                                                  uri, display_name, startup_id,
+                                                  (xfdesktop_file_manager_proxy_launch_reply)xfdesktop_file_utils_async_cb,
+                                                  parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
@@ -1171,31 +1194,6 @@ xfdesktop_file_utils_execute(GFile *working_directory,
             uris[n] = g_file_get_uri(lp->data);
         uris[n] = NULL;
 
-        /* If the working_dir wasn't set check if this is a .desktop file
-         * we can parse a working dir from */
-        if(working_dir == NULL) {
-            GFileInfo *info = g_file_query_info(file,
-                                                XFDESKTOP_FILE_INFO_NAMESPACE,
-                                                G_FILE_QUERY_INFO_NONE,
-                                                NULL, NULL);
-
-            if(xfdesktop_file_utils_is_desktop_file(info)) {
-                XfceRc *rc;
-                gchar *path = g_file_get_path(file);
-                if(path != NULL) {
-                    rc = xfce_rc_simple_open(path, TRUE);
-                    if(rc != NULL) {
-                        working_dir = g_strdup(xfce_rc_read_entry(rc, "Path", NULL));
-                        xfce_rc_close(rc);
-                    }
-                    g_free(path);
-                }
-            }
-
-            if(info)
-                g_object_unref(info);
-        }
-
         if(!xfdesktop_file_manager_proxy_execute(fileman_proxy,
                                                  working_dir, uri,
                                                  (const gchar **)uris,
@@ -1264,27 +1262,18 @@ xfdesktop_file_utils_display_chooser_dialog(GFile *file,
 
     fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
     if(fileman_proxy) {
-        GError *error = NULL;
         gchar *uri = g_file_get_uri(file);
         gchar *display_name = gdk_screen_make_display_name(screen);
         gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
 
-        if(!xfdesktop_file_manager_proxy_display_chooser_dialog(fileman_proxy,
-                                                                uri, open,
-                                                                display_name,
-                                                                startup_id,
-                                                                &error))
-        {
-            xfce_message_dialog(parent,
-                                _("Launch Error"), GTK_STOCK_DIALOG_ERROR,
-                                _("The application chooser could not be opened"),
-                                error->message, GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
-                                NULL);
-
-            g_error_free(error);
-        }
+        xfdesktop_file_manager_proxy_display_chooser_dialog_async(fileman_proxy,
+                                                                  uri, open,
+                                                                  display_name,
+                                                                  startup_id,
+                                                                  (xfdesktop_file_manager_proxy_display_chooser_dialog_reply)xfdesktop_file_utils_async_cb,
+                                                                  parent);
 
         xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
 
