@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifndef __XFDESKTOP_ICON_H__
@@ -63,15 +63,18 @@ struct _XfdesktopIconClass
     gboolean (*activated)(XfdesktopIcon *icon);
     
     /*< virtual functions >*/
-    GdkPixbuf *(*peek_pixbuf)(XfdesktopIcon *icon, gint size);
+    GdkPixbuf *(*peek_pixbuf)(XfdesktopIcon *icon, gint width, gint height);
     G_CONST_RETURN gchar *(*peek_label)(XfdesktopIcon *icon);
     
     GdkDragAction (*get_allowed_drag_actions)(XfdesktopIcon *icon);
     
     GdkDragAction (*get_allowed_drop_actions)(XfdesktopIcon *icon, GdkDragAction *suggested_action);
     gboolean (*do_drop_dest)(XfdesktopIcon *icon, XfdesktopIcon *src_icon, GdkDragAction action);
-    
+
+    GdkPixbuf *(*peek_tooltip_pixbuf)(XfdesktopIcon *icon, gint width, gint height);
     G_CONST_RETURN gchar *(*peek_tooltip)(XfdesktopIcon *icon);
+
+    gchar *(*get_identifier)(XfdesktopIcon *icon);
     
     void (*set_thumbnail_file)(XfdesktopIcon *icon, GFile *file);
     void (*delete_thumbnail_file)(XfdesktopIcon *icon);
@@ -85,10 +88,17 @@ GType xfdesktop_icon_get_type(void) G_GNUC_CONST;
 /* xfdesktop virtual function accessors */
 
 GdkPixbuf *xfdesktop_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                     gint size);
+                                     gint width,
+                                     gint height);
 G_CONST_RETURN gchar *xfdesktop_icon_peek_label(XfdesktopIcon *icon);
+GdkPixbuf *xfdesktop_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                              gint width,
+                                              gint height);
 G_CONST_RETURN gchar *xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon);
 
+/* returns a unique identifier for the icon, free when done using it */
+gchar *xfdesktop_icon_get_identifier(XfdesktopIcon *icon);
+
 void xfdesktop_icon_set_position(XfdesktopIcon *icon,
                                  gint16 row,
                                  gint16 col);
@@ -112,6 +122,10 @@ GtkWidget *xfdesktop_icon_peek_icon_view(XfdesktopIcon *icon);
 void xfdesktop_icon_set_thumbnail_file(XfdesktopIcon *icon, GFile *file);
 void xfdesktop_icon_delete_thumbnail(XfdesktopIcon *icon);
 
+void xfdesktop_icon_invalidate_regular_pixbuf(XfdesktopIcon *icon);
+void xfdesktop_icon_invalidate_tooltip_pixbuf(XfdesktopIcon *icon);
+void xfdesktop_icon_invalidate_pixbuf(XfdesktopIcon *icon);
+
 /*< signal triggers >*/
 
 void xfdesktop_icon_pixbuf_changed(XfdesktopIcon *icon);
