@@ -16,8 +16,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,14 +55,13 @@
 #include "xfdesktop-regular-file-icon.h"
 
 #define EMBLEM_SYMLINK  "emblem-symbolic-link"
+#define EMBLEM_READONLY "emblem-readonly"
 
 struct _XfdesktopRegularFileIconPrivate
 {
-    GdkPixbuf *pix;
     gchar *display_name;
     gchar *tooltip;
     guint pix_opacity;
-    gint cur_pix_size;
     GFileInfo *file_info;
     GFileInfo *filesystem_info;
     GFile *file;
@@ -76,8 +75,11 @@ static void xfdesktop_regular_file_icon_set_thumbnail_file(XfdesktopIcon *icon,
 static void xfdesktop_regular_file_icon_delete_thumbnail_file(XfdesktopIcon *icon);
 
 static GdkPixbuf *xfdesktop_regular_file_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                                          gint size);
+                                                          gint width, gint height);
 static G_CONST_RETURN gchar *xfdesktop_regular_file_icon_peek_label(XfdesktopIcon *icon);
+static gchar *xfdesktop_regular_file_icon_get_identifier(XfdesktopIcon *icon);
+static GdkPixbuf *xfdesktop_regular_file_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                                                  gint width, gint height);
 static G_CONST_RETURN gchar *xfdesktop_regular_file_icon_peek_tooltip(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_regular_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_regular_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
@@ -95,12 +97,7 @@ static gboolean xfdesktop_regular_file_can_write_parent(XfdesktopFileIcon *icon)
 
 #ifdef HAVE_THUNARX
 static void xfdesktop_regular_file_icon_tfi_init(ThunarxFileInfoIface *iface);
-#endif
 
-static inline void xfdesktop_regular_file_icon_invalidate_pixbuf(XfdesktopRegularFileIcon *icon);
-
-
-#ifdef HAVE_THUNARX
 G_DEFINE_TYPE_EXTENDED(XfdesktopRegularFileIcon, xfdesktop_regular_file_icon,
                        XFDESKTOP_TYPE_FILE_ICON, 0,
                        G_IMPLEMENT_INTERFACE(THUNARX_TYPE_FILE_INFO,
@@ -126,6 +123,8 @@ xfdesktop_regular_file_icon_class_init(XfdesktopRegularFileIconClass *klass)
     
     icon_class->peek_pixbuf = xfdesktop_regular_file_icon_peek_pixbuf;
     icon_class->peek_label = xfdesktop_regular_file_icon_peek_label;
+    icon_class->get_identifier = xfdesktop_regular_file_icon_get_identifier;
+    icon_class->peek_tooltip_pixbuf = xfdesktop_regular_file_icon_peek_tooltip_pixbuf;
     icon_class->peek_tooltip = xfdesktop_regular_file_icon_peek_tooltip;
     icon_class->get_allowed_drag_actions = xfdesktop_regular_file_icon_get_allowed_drag_actions;
     icon_class->get_allowed_drop_actions = xfdesktop_regular_file_icon_get_allowed_drop_actions;
@@ -158,11 +157,8 @@ xfdesktop_regular_file_icon_finalize(GObject *obj)
     GtkIconTheme *itheme = gtk_icon_theme_get_for_screen(icon->priv->gscreen);
     
     g_signal_handlers_disconnect_by_func(G_OBJECT(itheme),
-                                         G_CALLBACK(xfdesktop_regular_file_icon_invalidate_pixbuf),
+                                         G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
                                          icon);
-
-    if(icon->priv->pix)
-        g_object_unref(G_OBJECT(icon->priv->pix));
     
     if(icon->priv->file_info)
         g_object_unref(icon->priv->file_info);
@@ -173,7 +169,7 @@ xfdesktop_regular_file_icon_finalize(GObject *obj)
     g_object_unref(icon->priv->file);
     
     g_free(icon->priv->display_name);
-    
+
     if(icon->priv->tooltip)
         g_free(icon->priv->tooltip);
 
@@ -200,15 +196,6 @@ xfdesktop_regular_file_icon_tfi_init(ThunarxFileInfoIface *iface)
 }
 #endif
 
-static inline void
-xfdesktop_regular_file_icon_invalidate_pixbuf(XfdesktopRegularFileIcon *icon)
-{
-    if(icon->priv->pix) {
-        g_object_unref(G_OBJECT(icon->priv->pix));
-        icon->priv->pix = NULL;
-    }
-}
-
 static void
 xfdesktop_regular_file_icon_delete_thumbnail_file(XfdesktopIcon *icon)
 {
@@ -224,8 +211,10 @@ xfdesktop_regular_file_icon_delete_thumbnail_file(XfdesktopIcon *icon)
         file_icon->priv->thumbnail_file = NULL;
     }
 
-    xfdesktop_regular_file_icon_invalidate_pixbuf(file_icon);
-    xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
+    xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(icon));
+
+    xfdesktop_icon_invalidate_pixbuf(icon);
+    xfdesktop_icon_pixbuf_changed(icon);
 }
 
 static void
@@ -243,124 +232,154 @@ xfdesktop_regular_file_icon_set_thumbnail_file(XfdesktopIcon *icon, GFile *file)
 
     file_icon->priv->thumbnail_file = file;
 
-    xfdesktop_regular_file_icon_invalidate_pixbuf(file_icon);
-    xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
+    xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(icon));
+
+    xfdesktop_icon_invalidate_pixbuf(icon);
+    xfdesktop_icon_pixbuf_changed(icon);
+}
+
+static GIcon *
+xfdesktop_load_icon_from_desktop_file(XfdesktopRegularFileIcon *regular_icon)
+{
+    gchar *contents;
+    gsize length;
+    GIcon *gicon = NULL;
+
+    /* try to load the file into memory */
+    if(g_file_load_contents(regular_icon->priv->file, NULL, &contents, &length,
+                            NULL, NULL))
+    {
+        /* allocate a new key file */
+        GKeyFile *key_file = g_key_file_new();
+
+        /* try to parse the key file from the contents of the file */
+        if (g_key_file_load_from_data(key_file, contents, length, 0, NULL)) {
+            gchar *icon_name;
+            /* try to determine the custom icon name */
+            icon_name = g_key_file_get_string(key_file,
+                                              G_KEY_FILE_DESKTOP_GROUP,
+                                              G_KEY_FILE_DESKTOP_KEY_ICON,
+                                              NULL);
+
+            /* Create the themed icon for it */
+            if(icon_name) {
+                gicon = g_themed_icon_new(icon_name);
+                g_free(icon_name);
+            }
+        }
+
+        /* free key file and in-memory data */
+        g_key_file_free(key_file);
+        g_free(contents);
+    }
+
+    return gicon;
+}
+
+static GIcon *
+xfdesktop_regular_file_icon_load_icon(XfdesktopIcon *icon)
+{
+    XfdesktopRegularFileIcon *regular_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
+    XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(icon);
+    GIcon *gicon = NULL;
+
+    TRACE("entering");
+
+    /* Try to load the icon referenced in the .desktop file */
+    if(xfdesktop_file_utils_is_desktop_file(regular_icon->priv->file_info)) {
+        gicon = xfdesktop_load_icon_from_desktop_file(regular_icon);
+    } else {
+        /* If we have a thumbnail then they are enabled, use it. */
+        if(regular_icon->priv->thumbnail_file) {
+            gchar *file = g_file_get_path(regular_icon->priv->file);
+            gchar *mimetype = xfdesktop_get_file_mimetype(file);
+
+            /* Don't use thumbnails for svg, use the file itself */
+            if(g_strcmp0(mimetype, "image/svg+xml") == 0)
+                gicon = g_file_icon_new(regular_icon->priv->file);
+            else
+                gicon = g_file_icon_new(regular_icon->priv->thumbnail_file);
+
+            g_free(mimetype);
+            g_free(file);
+        }
+    }
+
+    /* If we still don't have an icon, use the default */
+    if(!G_IS_ICON(gicon)) {
+        gicon = g_file_info_get_icon(regular_icon->priv->file_info);
+        if(G_IS_ICON(gicon))
+            g_object_ref(gicon);
+    }
+
+    g_object_set(file_icon, "gicon", gicon, NULL);
+
+    /* Add any user set emblems */
+    gicon = xfdesktop_file_icon_add_emblems(file_icon);
+
+    /* load the read only emblem if necessary */
+    if(!g_file_info_get_attribute_boolean(regular_icon->priv->file_info,
+                                          G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+    {
+        GIcon *themed_icon = g_themed_icon_new(EMBLEM_READONLY);
+        GEmblem *emblem = g_emblem_new(themed_icon);
+
+        g_emblemed_icon_add_emblem(G_EMBLEMED_ICON(gicon), emblem);
+
+        g_object_unref(emblem);
+        g_object_unref(themed_icon);
+    }
+
+    /* load the symlink emblem if necessary */
+    if(g_file_info_get_attribute_boolean(regular_icon->priv->file_info,
+                                         G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK))
+    {
+        GIcon *themed_icon = g_themed_icon_new(EMBLEM_SYMLINK);
+        GEmblem *emblem = g_emblem_new(themed_icon);
+
+        g_emblemed_icon_add_emblem(G_EMBLEMED_ICON(gicon), emblem);
+
+        g_object_unref(emblem);
+        g_object_unref(themed_icon);
+    }
+
+    return gicon;
 }
 
 static GdkPixbuf *
 xfdesktop_regular_file_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                        gint size)
+                                        gint width, gint height)
 {
-    XfdesktopRegularFileIcon *file_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
-    gchar *icon_name = NULL;
-    GdkPixbuf *emblem_pix = NULL;
+    XfdesktopRegularFileIcon *regular_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
+    GIcon *gicon = NULL;
+    GdkPixbuf *pix = NULL;
 
-    if(size != file_icon->priv->cur_pix_size)
-        xfdesktop_regular_file_icon_invalidate_pixbuf(file_icon);
+    if(!xfdesktop_file_icon_has_gicon(XFDESKTOP_FILE_ICON(icon)))
+        gicon = xfdesktop_regular_file_icon_load_icon(icon);
+    else
+        g_object_get(XFDESKTOP_FILE_ICON(icon), "gicon", &gicon, NULL);
 
-    if(!file_icon->priv->pix) {
-        GIcon *gicon = NULL;
+    pix = xfdesktop_file_utils_get_icon(gicon, width, height,
+                                        regular_icon->priv->pix_opacity);
 
-        /* create a GFile for the $HOME/.thumbnails/ directory */
-        gchar *thumbnail_dir_path = g_build_filename(xfce_get_homedir(), 
-                                                     ".thumbnails", NULL);
-        GFile *thumbnail_dir = g_file_new_for_path(thumbnail_dir_path);
+    return pix;
+}
 
-        if(g_file_has_prefix(file_icon->priv->file, thumbnail_dir)) {
-            /* use the filename as custom icon name for thumbnails */
-            icon_name = g_file_get_path(file_icon->priv->file);
+static GdkPixbuf *
+xfdesktop_regular_file_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                                gint width, gint height)
+{
+    GIcon *gicon = NULL;
+    GdkPixbuf *tooltip_pix = NULL;
 
-            /* release thumbnail path */
-            g_object_unref(thumbnail_dir);
-            g_free(thumbnail_dir_path);
-        } else if(xfdesktop_file_utils_is_desktop_file(file_icon->priv->file_info)) {
-            gchar *contents;
-            gsize length;
+    if(!xfdesktop_file_icon_has_gicon(XFDESKTOP_FILE_ICON(icon)))
+        gicon = xfdesktop_regular_file_icon_load_icon(icon);
+    else
+        g_object_get(XFDESKTOP_FILE_ICON(icon), "gicon", &gicon, NULL);
 
-            /* try to load the file into memory */
-            if(g_file_load_contents(file_icon->priv->file, NULL, &contents, &length, 
-                                    NULL, NULL)) 
-            {
-                /* allocate a new key file */
-                GKeyFile *key_file = g_key_file_new();
+    tooltip_pix = xfdesktop_file_utils_get_icon(gicon, width, height, 100);
 
-                /* try to parse the key file from the contents of the file */
-                if (g_key_file_load_from_data(key_file, contents, length, 0, NULL)) {
-                    /* try to determine the custom icon name */
-                    icon_name = g_key_file_get_string(key_file,
-                                                      G_KEY_FILE_DESKTOP_GROUP,
-                                                      G_KEY_FILE_DESKTOP_KEY_ICON,
-                                                      NULL);
-                }
-
-                /* free key file and in-memory data */
-                g_key_file_free(key_file);
-                g_free(contents);
-            }
-        } else {
-            /* If we have a thumbnail then they are enabled, use it. */
-            if(file_icon->priv->thumbnail_file)
-            {
-                file_icon->priv->pix = gdk_pixbuf_new_from_file_at_size(g_file_get_path(file_icon->priv->thumbnail_file),
-                                                                        size, size,
-                                                                        NULL);
-            }
-        }
-
-        /* load the symlink emblem if necessary */
-        if(g_file_info_get_attribute_boolean(file_icon->priv->file_info, 
-                                             G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK)) 
-        {
-            GtkIconTheme *itheme = gtk_icon_theme_get_default();
-            gint sym_pix_size = size * 2 / 3;
-            
-            emblem_pix = gtk_icon_theme_load_icon(itheme, EMBLEM_SYMLINK,
-                                                  sym_pix_size, ITHEME_FLAGS,
-                                                  NULL);
-            if(emblem_pix) {
-                if(gdk_pixbuf_get_width(emblem_pix) != sym_pix_size
-                   || gdk_pixbuf_get_height(emblem_pix) != sym_pix_size)
-                {
-                    GdkPixbuf *tmp = gdk_pixbuf_scale_simple(emblem_pix,
-                                                             sym_pix_size,
-                                                             sym_pix_size,
-                                                             GDK_INTERP_BILINEAR);
-                    g_object_unref(emblem_pix);
-                    emblem_pix = tmp;
-                }
-            }
-        }
-
-        if(file_icon->priv->file_info)
-            gicon = g_file_info_get_icon(file_icon->priv->file_info);
-
-        if(file_icon->priv->pix) {
-            if(emblem_pix) {
-                gint emblem_pix_size = gdk_pixbuf_get_width(emblem_pix);
-                gint dest_size = size - emblem_pix_size;
-
-                /* We have to add the emblem */
-                gdk_pixbuf_composite(emblem_pix, file_icon->priv->pix,
-                                     dest_size, dest_size,
-                                     emblem_pix_size, emblem_pix_size,
-                                     dest_size, dest_size,
-                                     1.0, 1.0, GDK_INTERP_BILINEAR, 255);
-            }
-        } else {
-            file_icon->priv->pix = xfdesktop_file_utils_get_icon(icon_name, gicon,
-                                                                 size, emblem_pix,
-                                                                 file_icon->priv->pix_opacity);
-        }
-        
-        file_icon->priv->cur_pix_size = size;
-
-        if(emblem_pix)
-             g_object_unref(emblem_pix);
-        
-        g_free(icon_name);
-    }
-    
-    return file_icon->priv->pix;
+    return tooltip_pix;
 }
 
 static G_CONST_RETURN gchar *
@@ -373,6 +392,19 @@ xfdesktop_regular_file_icon_peek_label(XfdesktopIcon *icon)
     return regular_file_icon->priv->display_name;
 }
 
+static gchar *
+xfdesktop_regular_file_icon_get_identifier(XfdesktopIcon *icon)
+{
+    XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(icon);
+
+    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(icon), NULL);
+
+    if(xfdesktop_file_icon_peek_file(file_icon) == NULL)
+        return NULL;
+
+    return g_file_get_path(xfdesktop_file_icon_peek_file(file_icon));
+}
+
 static GdkDragAction
 xfdesktop_regular_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 {
@@ -702,7 +734,8 @@ xfdesktop_regular_file_icon_update_file_info(XfdesktopFileIcon *icon,
     regular_file_icon->priv->tooltip = NULL;
     
     /* not really easy to check if this changed or not, so just invalidate it */
-    xfdesktop_regular_file_icon_invalidate_pixbuf(regular_file_icon);
+    xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(icon));
+    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(icon));
     xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
 }
 
@@ -744,7 +777,7 @@ xfdesktop_regular_file_icon_new(GFile *file,
 
     g_signal_connect_swapped(G_OBJECT(gtk_icon_theme_get_for_screen(screen)),
                              "changed",
-                             G_CALLBACK(xfdesktop_regular_file_icon_invalidate_pixbuf),
+                             G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
                              regular_file_icon);
     
     return regular_file_icon;
@@ -761,6 +794,6 @@ xfdesktop_regular_file_icon_set_pixbuf_opacity(XfdesktopRegularFileIcon *icon,
     
     icon->priv->pix_opacity = opacity;
     
-    xfdesktop_regular_file_icon_invalidate_pixbuf(icon);
+    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(icon));
     xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
 }
