@@ -16,8 +16,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,9 +60,7 @@
 
 struct _XfdesktopVolumeIconPrivate
 {
-    GdkPixbuf *pix;
     gchar *tooltip;
-    gint cur_pix_size;
     gchar *label;
     GVolume *volume;
     GFileInfo *file_info;
@@ -77,8 +75,11 @@ struct _XfdesktopVolumeIconPrivate
 static void xfdesktop_volume_icon_finalize(GObject *obj);
 
 static GdkPixbuf *xfdesktop_volume_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                                    gint size);
+                                                    gint width, gint height);
 static G_CONST_RETURN gchar *xfdesktop_volume_icon_peek_label(XfdesktopIcon *icon);
+static gchar *xfdesktop_volume_icon_get_identifier(XfdesktopIcon *icon);
+static GdkPixbuf *xfdesktop_volume_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                                            gint width, gint height);
 static G_CONST_RETURN gchar *xfdesktop_volume_icon_peek_tooltip(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_volume_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_volume_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
@@ -101,12 +102,7 @@ static void xfdesktop_volume_icon_changed(GVolume *volume,
 
 #ifdef HAVE_THUNARX
 static void xfdesktop_volume_icon_tfi_init(ThunarxFileInfoIface *iface);
-#endif
 
-static inline void xfdesktop_volume_icon_invalidate_pixbuf(XfdesktopVolumeIcon *icon);
-
-
-#ifdef HAVE_THUNARX
 G_DEFINE_TYPE_EXTENDED(XfdesktopVolumeIcon, xfdesktop_volume_icon,
                        XFDESKTOP_TYPE_FILE_ICON, 0,
                        G_IMPLEMENT_INTERFACE(THUNARX_TYPE_FILE_INFO,
@@ -138,6 +134,8 @@ xfdesktop_volume_icon_class_init(XfdesktopVolumeIconClass *klass)
     
     icon_class->peek_pixbuf = xfdesktop_volume_icon_peek_pixbuf;
     icon_class->peek_label = xfdesktop_volume_icon_peek_label;
+    icon_class->get_identifier = xfdesktop_volume_icon_get_identifier;
+    icon_class->peek_tooltip_pixbuf = xfdesktop_volume_icon_peek_tooltip_pixbuf;
     icon_class->peek_tooltip = xfdesktop_volume_icon_peek_tooltip;
     icon_class->get_allowed_drag_actions = xfdesktop_volume_icon_get_allowed_drag_actions;
     icon_class->get_allowed_drop_actions = xfdesktop_volume_icon_get_allowed_drop_actions;
@@ -171,17 +169,14 @@ xfdesktop_volume_icon_finalize(GObject *obj)
         g_source_remove(icon->priv->changed_timeout_id);
     
     g_signal_handlers_disconnect_by_func(G_OBJECT(itheme),
-                                         G_CALLBACK(xfdesktop_volume_icon_invalidate_pixbuf),
+                                         G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
                                          icon);
     
     if(icon->priv->label) {
         g_free(icon->priv->label);
         icon->priv->label = NULL;
     }
-    
-    if(icon->priv->pix)
-        g_object_unref(G_OBJECT(icon->priv->pix));
-    
+
     if(icon->priv->file_info)
         g_object_unref(icon->priv->file_info);
 
@@ -193,7 +188,7 @@ xfdesktop_volume_icon_finalize(GObject *obj)
 
     if(icon->priv->volume)
         g_object_unref(G_OBJECT(icon->priv->volume));
-    
+
     if(icon->priv->tooltip)
         g_free(icon->priv->tooltip);
     
@@ -218,15 +213,6 @@ xfdesktop_volume_icon_tfi_init(ThunarxFileInfoIface *iface)
 #endif  /* HAVE_THUNARX */
 
 
-static inline void
-xfdesktop_volume_icon_invalidate_pixbuf(XfdesktopVolumeIcon *icon)
-{
-    if(icon->priv->pix) {
-        g_object_unref(G_OBJECT(icon->priv->pix));
-        icon->priv->pix = NULL;
-    }
-}
-
 static gboolean
 xfdesktop_volume_icon_is_mounted(XfdesktopIcon *icon)
 {
@@ -252,41 +238,72 @@ xfdesktop_volume_icon_is_mounted(XfdesktopIcon *icon)
     return ret;
 }
 
+static GIcon *
+xfdesktop_volume_icon_load_icon(XfdesktopIcon *icon)
+{
+    XfdesktopVolumeIcon *volume_icon = XFDESKTOP_VOLUME_ICON(icon);
+    XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(icon);
+    GIcon *gicon = NULL;
+
+    TRACE("entering");
+
+    /* load icon and keep a ref to it */
+    if(volume_icon->priv->volume) {
+        gicon = g_volume_get_icon(volume_icon->priv->volume);
+
+        if(G_IS_ICON(gicon))
+            g_object_ref(gicon);
+
+        g_object_set(file_icon, "gicon", gicon, NULL);
+
+        /* Add any user set emblems */
+        gicon = xfdesktop_file_icon_add_emblems(file_icon);
+    }
+
+    return gicon;
+}
+
 static GdkPixbuf *
 xfdesktop_volume_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                  gint size)
+                                  gint width, gint height)
 {
-    XfdesktopVolumeIcon *file_icon = XFDESKTOP_VOLUME_ICON(icon);
+    gint opacity = 100;
+    GIcon *gicon = NULL;
+    GdkPixbuf *pix = NULL;
     
     g_return_val_if_fail(XFDESKTOP_IS_VOLUME_ICON(icon), NULL);
-    
-    if(size != file_icon->priv->cur_pix_size)
-        xfdesktop_volume_icon_invalidate_pixbuf(file_icon);
 
-    if(!file_icon->priv->pix) {
-        GIcon *gicon = NULL;
+    if(!xfdesktop_file_icon_has_gicon(XFDESKTOP_FILE_ICON(icon)))
+        gicon = xfdesktop_volume_icon_load_icon(icon);
+    else
+        g_object_get(XFDESKTOP_FILE_ICON(icon), "gicon", &gicon, NULL);
 
-        if(file_icon->priv->volume)
-            gicon = g_volume_get_icon(file_icon->priv->volume);
+    /* If the volume isn't mounted show it as semi-transparent */
+    if(!xfdesktop_volume_icon_is_mounted(icon))
+        opacity = 50;
 
-        file_icon->priv->pix = xfdesktop_file_utils_get_icon(NULL, gicon, size, 
-                                                             NULL, 100);
+    pix = xfdesktop_file_utils_get_icon(gicon, height, height, opacity);
 
-        /* If the volume isn't mounted show it as semi-transparent */
-        if(!xfdesktop_volume_icon_is_mounted(icon)) {
-            GdkPixbuf *temp;
-            temp = exo_gdk_pixbuf_lucent(file_icon->priv->pix, 50);
+    return pix;
+}
 
-            if(temp != NULL) {
-                g_object_unref(G_OBJECT(file_icon->priv->pix));
-                file_icon->priv->pix = temp;
-            }
-        }
-        
-        file_icon->priv->cur_pix_size = size;
-    }
-    
-    return file_icon->priv->pix;
+static GdkPixbuf *
+xfdesktop_volume_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                          gint width, gint height)
+{
+    GIcon *gicon = NULL;
+    GdkPixbuf *tooltip_pix = NULL;
+
+    g_return_val_if_fail(XFDESKTOP_IS_VOLUME_ICON(icon), NULL);
+
+    if(!xfdesktop_file_icon_has_gicon(XFDESKTOP_FILE_ICON(icon)))
+        gicon = xfdesktop_volume_icon_load_icon(icon);
+    else
+        g_object_get(XFDESKTOP_FILE_ICON(icon), "gicon", &gicon, NULL);
+
+    tooltip_pix = xfdesktop_file_utils_get_icon(gicon, height, height, 100);
+
+    return tooltip_pix;
 }
 
 G_CONST_RETURN gchar *
@@ -303,6 +320,20 @@ xfdesktop_volume_icon_peek_label(XfdesktopIcon *icon)
     return volume_icon->priv->label;
 }
 
+static gchar *
+xfdesktop_volume_icon_get_identifier(XfdesktopIcon *icon)
+{
+    XfdesktopVolumeIcon *volume_icon = XFDESKTOP_VOLUME_ICON(icon);
+    gchar *uuid;
+
+    uuid = g_volume_get_identifier(volume_icon->priv->volume, G_VOLUME_IDENTIFIER_KIND_UUID);
+
+    if(uuid == NULL)
+        return g_strdup(xfdesktop_volume_icon_peek_label(icon));
+
+    return uuid;
+}
+
 static GdkDragAction
 xfdesktop_volume_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 {
@@ -476,12 +507,15 @@ xfdesktop_volume_icon_eject_finish(GObject *object,
     GtkWidget *toplevel = icon_view ? gtk_widget_get_toplevel(icon_view) : NULL;
     GVolume *volume = G_VOLUME(object);
     GError *error = NULL;
+    gboolean eject_successful;
       
     g_return_if_fail(G_IS_VOLUME(object));
     g_return_if_fail(G_IS_ASYNC_RESULT(result));
     g_return_if_fail(XFDESKTOP_IS_VOLUME_ICON(icon));
 
-    if(!g_volume_eject_with_operation_finish(volume, result, &error)) {
+    eject_successful = g_volume_eject_with_operation_finish(volume, result, &error);
+
+    if(!eject_successful) {
         /* ignore GIO errors handled internally */
         if(error->domain != G_IO_ERROR || error->code != G_IO_ERROR_FAILED_HANDLED) {
             gchar *volume_name = g_volume_get_name(volume);
@@ -502,7 +536,7 @@ xfdesktop_volume_icon_eject_finish(GObject *object,
     }
 
 #ifdef HAVE_LIBNOTIFY
-    xfdesktop_notify_eject_finish(volume);
+    xfdesktop_notify_eject_finish(volume, eject_successful);
 #endif
 
     g_object_unref(icon);
@@ -518,12 +552,15 @@ xfdesktop_volume_icon_unmount_finish(GObject *object,
     GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
     GMount *mount = G_MOUNT(object);
     GError *error = NULL;
+    gboolean unmount_successful;
       
     g_return_if_fail(G_IS_MOUNT(object));
     g_return_if_fail(G_IS_ASYNC_RESULT(result));
     g_return_if_fail(XFDESKTOP_IS_VOLUME_ICON(icon));
 
-    if(!g_mount_unmount_with_operation_finish(mount, result, &error)) {
+    unmount_successful = g_mount_unmount_with_operation_finish(mount, result, &error);
+
+    if(!unmount_successful) {
         /* ignore GIO errors handled internally */
         if(error->domain != G_IO_ERROR || error->code != G_IO_ERROR_FAILED_HANDLED) {
             gchar *mount_name = g_mount_get_name(mount);
@@ -544,7 +581,7 @@ xfdesktop_volume_icon_unmount_finish(GObject *object,
     }
 
 #ifdef HAVE_LIBNOTIFY
-    xfdesktop_notify_unmount_finish(mount);
+    xfdesktop_notify_unmount_finish(mount, unmount_successful);
 #endif
 
     g_object_unref(icon);
@@ -622,53 +659,108 @@ xfdesktop_volume_icon_mount_finish(GObject *object,
 }
 
 static void
-xfdesktop_volume_icon_menu_toggle_mount(GtkWidget *widget,
-                                        gpointer user_data)
+xfdesktop_volume_icon_menu_mount(GtkWidget *widget, gpointer user_data)
 {
     XfdesktopVolumeIcon *icon = XFDESKTOP_VOLUME_ICON(user_data);
     GtkWidget *icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(icon));
     GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
     GVolume *volume;
     GMount *mount;
-    
+    GMountOperation *operation;
+
     volume = xfdesktop_volume_icon_peek_volume(icon);
     mount = g_volume_get_mount(volume);
 
     if(mount) {
-        if(g_volume_can_eject(volume)) {
-#ifdef HAVE_LIBNOTIFY
-            xfdesktop_notify_eject(volume);
-#endif
-            /* TODO: GMountOperation could be used to show what processes
-             *       are preventing an eject. */
-            g_volume_eject_with_operation(volume, G_MOUNT_UNMOUNT_NONE,
-                                          NULL, NULL,
-                                          xfdesktop_volume_icon_eject_finish,
-                                          g_object_ref(icon));
-        } else {
-#ifdef HAVE_LIBNOTIFY
-            xfdesktop_notify_unmount(mount);
-#endif
-            /* TODO: GMountOperation could be used to show what processes
-             *       are preventing an unmount. */
-            g_mount_unmount_with_operation(mount, G_MOUNT_UNMOUNT_NONE,
-                                           NULL, NULL,
-                                           xfdesktop_volume_icon_unmount_finish,
-                                           g_object_ref(icon));
-        }
-    } else {
-        GMountOperation *operation;
+        g_object_unref(mount);
+        return;
+    }
 
+    operation = gtk_mount_operation_new(toplevel ? GTK_WINDOW(toplevel) : NULL);
+    gtk_mount_operation_set_screen(GTK_MOUNT_OPERATION(operation),
+                                   icon->priv->gscreen);
+
+    g_volume_mount(volume, G_MOUNT_MOUNT_NONE, operation, NULL,
+                   xfdesktop_volume_icon_mount_finish,
+                   g_object_ref(icon));
+
+    g_object_unref(operation);
+}
+
+static void
+xfdesktop_volume_icon_menu_unmount(GtkWidget *widget, gpointer user_data)
+{
+    XfdesktopVolumeIcon *icon = XFDESKTOP_VOLUME_ICON(user_data);
+    GtkWidget *icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(icon));
+    GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
+    GVolume *volume;
+    GMount *mount;
+    GMountOperation *operation;
+
+    volume = xfdesktop_volume_icon_peek_volume(icon);
+    mount = g_volume_get_mount(volume);
+
+    if(!mount)
+        return;
+
+#ifdef HAVE_LIBNOTIFY
+    xfdesktop_notify_unmount(mount);
+#endif
+
+    operation = gtk_mount_operation_new(toplevel ? GTK_WINDOW(toplevel) : NULL);
+    gtk_mount_operation_set_screen(GTK_MOUNT_OPERATION(operation),
+                                   icon->priv->gscreen);
+
+    g_mount_unmount_with_operation(mount,
+                                   G_MOUNT_UNMOUNT_NONE,
+                                   operation,
+                                   NULL,
+                                   xfdesktop_volume_icon_unmount_finish,
+                                   g_object_ref(icon));
+
+    g_object_unref(mount);
+    g_object_unref(operation);
+}
+
+static void
+xfdesktop_volume_icon_menu_eject(GtkWidget *widget,
+                                 gpointer user_data)
+{
+    XfdesktopVolumeIcon *icon = XFDESKTOP_VOLUME_ICON(user_data);
+    GtkWidget *icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(icon));
+    GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
+    GVolume *volume;
+    GMount *mount;
+    GMountOperation *operation = NULL;
+
+    volume = xfdesktop_volume_icon_peek_volume(icon);
+    mount = g_volume_get_mount(volume);
+
+    if(!mount)
+        return;
+
+    if(g_volume_can_eject(volume)) {
+#ifdef HAVE_LIBNOTIFY
+        xfdesktop_notify_eject(volume);
+#endif
         operation = gtk_mount_operation_new(toplevel ? GTK_WINDOW(toplevel) : NULL);
         gtk_mount_operation_set_screen(GTK_MOUNT_OPERATION(operation),
                                        icon->priv->gscreen);
 
-        g_volume_mount(volume, G_MOUNT_MOUNT_NONE, operation, NULL,
-                       xfdesktop_volume_icon_mount_finish, 
-                       g_object_ref(icon));
-
-        g_object_unref(operation);
+        g_volume_eject_with_operation(volume,
+                                      G_MOUNT_UNMOUNT_NONE,
+                                      operation,
+                                      NULL,
+                                      xfdesktop_volume_icon_eject_finish,
+                                      g_object_ref(icon));
+    } else {
+        /* If we can't eject the volume try to unmount it */
+        xfdesktop_volume_icon_menu_unmount(widget, user_data);
     }
+
+    g_object_unref(mount);
+    if(operation != NULL)
+        g_object_unref(operation);
 }
 
 static void
@@ -684,6 +776,30 @@ xfdesktop_volume_icon_menu_properties(GtkWidget *widget,
                                                 NULL);
 }
 
+static void
+xfdesktop_volume_icon_add_context_menu_option(XfdesktopIcon *icon,
+                                              const gchar* icon_name,
+                                              const gchar* icon_label,
+                                              GtkWidget *menu,
+                                              GCallback callback)
+{
+    GtkWidget *mi, *img;
+
+    img = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_MENU);
+    gtk_widget_show(img);
+    mi = gtk_image_menu_item_new_with_mnemonic(icon_label);
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+    gtk_widget_show(mi);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+    if(callback != NULL) {
+        g_signal_connect(G_OBJECT(mi), "activate",
+                         callback,
+                         icon);
+    } else {
+        gtk_widget_set_sensitive(mi, FALSE);
+    }
+}
+
 static gboolean
 xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
                                             GtkWidget *menu)
@@ -692,8 +808,11 @@ xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
     GVolume *volume = volume_icon->priv->volume;
     GtkWidget *mi, *img;
     GMount *mount;
-    const gchar *icon_name, *icon_label = NULL;
-    
+    const gchar *icon_name, *icon_label;
+
+    icon_name = GTK_STOCK_OPEN;
+    icon_label = _("_Open");
+
     img = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
     gtk_widget_show(img);
     mi = gtk_image_menu_item_new_with_mnemonic(_("_Open"));
@@ -709,51 +828,43 @@ xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
     
     mount = g_volume_get_mount(volume);
 
-    if(mount) {
-        if(g_volume_can_eject(volume)) {
-            icon_name = "media-eject";
-            icon_label = _("E_ject Volume");
-        } else if(g_mount_can_unmount(mount)) {
-            icon_name = "media-eject";
-            icon_label = _("E_ject Volume");
-        }
+    if(mount && g_volume_can_eject(volume)) {
+        icon_name = "media-eject";
+        icon_label = _("E_ject Volume");
+        xfdesktop_volume_icon_add_context_menu_option(icon, icon_name, icon_label,
+                        menu, G_CALLBACK(xfdesktop_volume_icon_menu_eject));
+    }
 
+    if(mount && g_mount_can_unmount(mount)) {
+        icon_name = NULL;
+        icon_label = _("_Unmount Volume");
+        xfdesktop_volume_icon_add_context_menu_option(icon, icon_name, icon_label,
+                        menu, G_CALLBACK(xfdesktop_volume_icon_menu_unmount));
+    }
+
+    if(!mount && g_volume_can_mount(volume)) {
+        icon_name = NULL;
+        icon_label = _("_Mount Volume");
+        xfdesktop_volume_icon_add_context_menu_option(icon, icon_name, icon_label,
+                        menu, G_CALLBACK(xfdesktop_volume_icon_menu_mount));
+    }
+
+    if(mount)
         g_object_unref(mount);
-    } else {
-        if(g_volume_can_mount(volume)) {
-            icon_name = NULL;
-            icon_label = _("_Mount Volume");
-        }
-    }
-
-    if(xfdesktop_file_utils_volume_is_removable(volume) && icon_label) {
-        img = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_MENU);
-        gtk_widget_show(img);
-        mi = gtk_image_menu_item_new_with_mnemonic(icon_label);
-        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
-        gtk_widget_show(mi);
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-        g_signal_connect(G_OBJECT(mi), "activate",
-                         G_CALLBACK(xfdesktop_volume_icon_menu_toggle_mount),
-                         icon);
-    }
 
     mi = gtk_separator_menu_item_new();
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
 
-    img = gtk_image_new_from_stock(GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU);
-    gtk_widget_show(img);
-    mi = gtk_image_menu_item_new_with_mnemonic(_("P_roperties..."));
-    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+    icon_name = GTK_STOCK_PROPERTIES;
+    icon_label = _("P_roperties...");
+
     if(!volume_icon->priv->file_info)
-        gtk_widget_set_sensitive(mi, FALSE);
+        xfdesktop_volume_icon_add_context_menu_option(icon, icon_name, icon_label,
+                                                      menu, NULL);
     else {
-        g_signal_connect(G_OBJECT(mi), "activate",
-                         G_CALLBACK(xfdesktop_volume_icon_menu_properties),
-                         icon);
+        xfdesktop_volume_icon_add_context_menu_option(icon, icon_name, icon_label,
+                    menu, G_CALLBACK(xfdesktop_volume_icon_menu_properties));
     }
     
     return TRUE;
@@ -812,7 +923,8 @@ xfdesktop_volume_icon_update_file_info(XfdesktopFileIcon *icon,
     }
 
     /* not really easy to check if this changed or not, so just invalidate it */
-    xfdesktop_volume_icon_invalidate_pixbuf(volume_icon);
+    xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(icon));
+    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(icon));
     xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
 }
 
@@ -834,7 +946,7 @@ xfdesktop_volume_icon_activated(XfdesktopIcon *icon_p)
                            GUINT_TO_POINTER(TRUE));
 
         /* mount the volume and open the folder in the mount finish callback */
-        xfdesktop_volume_icon_menu_toggle_mount(NULL, icon);
+        xfdesktop_volume_icon_menu_mount(NULL, icon);
 
         return TRUE;
     } else {
@@ -907,7 +1019,7 @@ volume_icon_changed_timeout(XfdesktopVolumeIcon *volume_icon)
         }
 
         /* not really easy to check if this changed or not, so just invalidate it */
-        xfdesktop_volume_icon_invalidate_pixbuf(volume_icon);
+        xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(volume_icon));
         xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(volume_icon));
 
         /* finalize the timeout source */
@@ -1002,7 +1114,7 @@ xfdesktop_volume_icon_new(GVolume *volume,
 
     g_signal_connect_swapped(G_OBJECT(gtk_icon_theme_get_for_screen(screen)),
                              "changed",
-                             G_CALLBACK(xfdesktop_volume_icon_invalidate_pixbuf),
+                             G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
                              volume_icon);
 
     g_signal_connect(volume, "changed", 
